﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Components;
using Rhino.Mocks;
using Buildcraft.Core.Exceptions;
using Buildcraft.Core.Interfaces;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.UnitTasks;

namespace Buildcraft.Core.Test.ComponentTests
{
    [TestClass]
    public class BasicTaskSchedulerTests
    {
        [TestMethod]
        [ExpectedException(typeof(BuildcraftException))]
        public void BasicTaskSchedulerTests_CannotCreateWithZeroFrameLength()
        {
            BasicTaskScheduler scheduler = new BasicTaskScheduler(0);
        }

        [TestMethod]
        [ExpectedException(typeof(BuildcraftException))]
        public void BasicTaskSchedulerTests_CannotCreateWithNegativeFrameLength()
        {
            BasicTaskScheduler scheduler = new BasicTaskScheduler(-1);
        }

        [TestMethod]
        public void BasicTaskSchedulerTests_RegisterTickFrameUnregisterProcessor()
        {
            BasicTaskScheduler scheduler = new BasicTaskScheduler(100);
            ITaskProcessor processor = MockRepository.GenerateStub<ITaskProcessor>();
            scheduler.RegisterTaskProcessor(processor);

            //Incomplete Tick - Assert was not called
            scheduler.Tick(99);
            processor.AssertWasNotCalled(p => p.Process());

            //Complete Tick - Assert was called 
            scheduler.Tick(1);
            processor.AssertWasCalled(p => p.Process(), options => options.Repeat.Once());

            //Remove processor and tick again - Assert was not called (second time)
            scheduler.UnregisterTaskProcessor(processor);
            scheduler.Tick(100);
            processor.AssertWasCalled(p => p.Process(), options => options.Repeat.Once());
        }

        [TestMethod]
        public void BasicTaskSchedulerTests_RegisterProcessorRegisterUnregisterBucket()
        {
            //Setup
            BasicTaskScheduler scheduler = new BasicTaskScheduler(100);
            ITaskProcessor processor = MockRepository.GenerateStub<ITaskProcessor>();
            IUnitTaskStateBucket bucket = MockRepository.GenerateStub<IUnitTaskStateBucket>();
            scheduler.RegisterTaskProcessor(processor);

            //Register and Check
            scheduler.RegisterStateBucket(bucket);
            processor.AssertWasCalled(p => p.RegisterStateBucket(bucket));

            //Unregister and check
            scheduler.UnregisterStateBucket(bucket);
            processor.AssertWasCalled(p => p.UnregisterStateBucket(bucket));
        }

        [TestMethod]
        public void BasicTaskSchedulerTests_RegisterProcessorRegisterUnregisterState()
        {
            //Setup
            BasicTaskScheduler scheduler = new BasicTaskScheduler(100);
            ITaskProcessor processor = MockRepository.GenerateStub<ITaskProcessor>();
            IUnitTaskStateItem state = MockRepository.GenerateStub<IUnitTaskStateItem>();
            scheduler.RegisterTaskProcessor(processor);

            //Register and Check
            scheduler.RegisterState(state);
            processor.AssertWasCalled(p => p.RegisterState(state));

            //Unregister and check
            scheduler.UnregisterState(state);
            processor.AssertWasCalled(p => p.UnregisterState(state));
        }
    }
}
