﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Interfaces;
using Buildcraft.Core.Components;
using Rhino.Mocks;

namespace Buildcraft.Core.Test.ComponentTests
{
    [TestClass]
    public class EntityCheckTests
    {
        private ITypeInfo _validType;
        private ITypeInfo _invalidType;

        private ITypedEntity<ITypeInfo> _validEntity;
        private ITypedEntity<ITypeInfo> _invalidEntity;

        [TestInitialize]
        public void Initialize()
        {
            _validType = MockRepository.GenerateMock<ITypeInfo>();
            _invalidType = MockRepository.GenerateMock<ITypeInfo>();

            _validEntity = MockRepository.GenerateMock<ITypedEntity<ITypeInfo>>();
            _invalidEntity = MockRepository.GenerateMock<ITypedEntity<ITypeInfo>>();

            _validEntity.Stub(e => e.TypeInfo).Return(_validType);
            _invalidEntity.Stub(e => e.TypeInfo).Return(_invalidType);
        }

        [TestMethod]
        public void ExclusiveSetTypeCheck()
        {
            var types = new List<ITypeInfo>();
            types.Add(_invalidType);

            var check = new ExclusiveSetCheck<ITypeInfo>(types);

            Assert.IsTrue(check.IsValid(_validEntity));
            Assert.IsFalse(check.IsValid(_invalidEntity));
        }

        [TestMethod]
        public void InclusiveSetTypeCheck()
        {
            var types = new List<ITypeInfo>();
            types.Add(_validType);

            var check = new InclusiveSetCheck<ITypeInfo>(types);

            Assert.IsTrue(check.IsValid(_validEntity));
            Assert.IsFalse(check.IsValid(_invalidEntity));
        }


        [TestMethod]
        public void SingleItemTypeCheck()
        {
            var check = new SingleTypeSetCheck<ITypeInfo>(_validType);

            Assert.IsTrue(check.IsValid(_validEntity));
            Assert.IsFalse(check.IsValid(_invalidEntity));
        }
    }
}
