﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Components;
using Buildcraft.Core.Model;
using Buildcraft.Core.Exceptions;
using Buildcraft.ScenarioBuilder;

namespace Buildcraft.Core.Test.ComponentTests
{
    [TestClass]
    public class WorldManagerTests : BaseTest
    {
        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
        }


        [TestMethod]
        public void WorldManagerTests_AddRemovePlayerTest()
        {
            var player1 = worldManager.AddPlayer("Player1");

            Assert.IsTrue(world.Players.Count == 1);
            Assert.IsTrue(world.Players[0] == player1);

            var player2 = worldManager.AddPlayer("Player2");

            Assert.IsTrue(world.Players.Count == 2);
            Assert.IsTrue(world.Players[1] == player2);

            worldManager.RemovePlayer(player1);

            Assert.IsTrue(world.Players.Count == 1);
            Assert.IsTrue(world.Players[0] == player2);
        }

        [TestMethod]
        public void WorldManagerTests_AddRemoveNodeTest()
        {
            var player = worldManager.AddPlayer("Player1");


            var node1 = worldManager.AddNode();

            Assert.IsTrue(world.Nodes.Count == 1);
            Assert.IsTrue(world.Nodes[0] == node1);


            var node2 = worldManager.AddNode();
            
            Assert.IsTrue(world.Nodes.Count == 2);
            Assert.IsTrue(world.Nodes[1] == node2);


            worldManager.RemoveNode(node1);

            Assert.IsTrue(world.Nodes.Count == 1);
            Assert.IsTrue(world.Nodes[0] == node2);
        }

        [TestMethod]
        public void WorldManagerTests_AddRemoveJoinTest()
        {
            var player = worldManager.AddPlayer("Player1");
            var node1 = worldManager.AddNode();
            var node2 = worldManager.AddNode();

            worldManager.AddJoin(node1, node2);

            Assert.IsTrue(node1.Joins.Count == 1);
            Assert.IsTrue(node1.Joins[0].FromNode == node1);
            Assert.IsTrue(node1.Joins[0].ToNode == node2);
            
            Assert.IsTrue(node2.Joins.Count == 1);
            Assert.IsTrue(node2.Joins[0].FromNode == node2);
            Assert.IsTrue(node2.Joins[0].ToNode == node1);

            worldManager.RemoveJoin(node1, node2);

            Assert.IsTrue(node1.Joins.Count == 0);
            Assert.IsTrue(node2.Joins.Count == 0);
        }


        [TestMethod]
        [ExpectedException(typeof(BuildcraftException))]
        public void WorldManagerTests_CannotAddDoubleJoin()
        {
            var node1 = worldManager.AddNode();
            var node2 = worldManager.AddNode();

            worldManager.AddJoin(node1, node2);
            worldManager.AddJoin(node1, node2);
        }


        [TestMethod]
        public void WorldManagerTests_SetClearNodeImprovement()
        {
            var node = worldManager.AddNode();
            var typeInfo = new ImprovementTypeInfo(ImprovementTypeCodes.City);

            worldManager.SetNodeImprovement(node, typeInfo);

            Assert.IsNotNull(node.Improvement);
            Assert.AreEqual(node.Improvement.TypeInfo, typeInfo);

            worldManager.ClearNodeImprovement(node);

            Assert.IsNull(node.Improvement);
        }

        [TestMethod]
        public void WorldManagerTests_SetClearNodeOwner()
        {
            var player = worldManager.AddPlayer("P1");
            var node = worldManager.AddNode();

            worldManager.SetNodeOwner(node, player);

            Assert.AreEqual(node.Owner, player);

            Assert.IsTrue(player.Nodes.Count == 1);
            Assert.IsTrue(player.Nodes[0] == node);

            worldManager.ClearNodeOwner(node);

            Assert.IsNull(node.Owner);
            Assert.IsTrue(player.Nodes.Count == 0);
        }
    }
}
