﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Model;

using Buildcraft.Core.NodeTasks.Conditions;
using Buildcraft.ScenarioBuilder;

namespace Buildcraft.Core.Test.ConditionTests
{
    [TestClass]
    public class AttackNodeConditionTests : BaseTest
    {
        //Model Parts
        private Node p1Node;
        private Node p2Node;

        private Player player1;
        private Player player2;
        
        private Worker worker;

        //TypeInfo's
        private ImprovementTypeInfo improvementTypeInfo;
        private ImprovementTypeInfo invalidImprovementTypeInfo;
        private WorkerTypeInfo workerTypeInfo;
        private WorkerTypeInfo invalidWorkerTypeInfo;

        //Condition
        private AttackNodeCondition condition = new AttackNodeCondition();

        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
            improvementTypeInfo = new ImprovementTypeInfo(ImprovementTypeCodes.City);
            invalidImprovementTypeInfo = new ImprovementTypeInfo(ImprovementTypeCodes.Farm);
            workerTypeInfo = new WorkerTypeInfo(WorkerTypeCodes.Generalist);
            invalidWorkerTypeInfo = new WorkerTypeInfo(WorkerTypeCodes.Administrator);

            condition.MinWorkers = 1;
            condition.MaxWorkers = 1;

            player1 = worldManager.AddPlayer("P1");
            player2 = worldManager.AddPlayer("P2");

            p1Node = base.CreateNode(player1, null);
            p2Node = base.CreateNode(player2, null);

            base.CreateJoin(p1Node, p2Node);

            worker = playerManager.AddWorker(player1, workerTypeInfo);

            worldManager.SetNodeOwner(p1Node, player1);
            worldManager.SetNodeImprovement(p1Node, improvementTypeInfo);
        }

        [TestMethod]
        public void AttackNodeConditionTests_Basic()
        {
            CheckNodeValidity();
            CheckWorkerValidity();
            CheckWorkersValidity();
        }

        private void CheckNodeValidity()
        {
            //Check Invalid In Friendly territory
            var result = condition.CheckNodeValidity(p1Node, player1);
            Assert.AreEqual(TaskValidityState.NotAllowedInFriendlyTerritory, result);

            //Check Valid in Enemy Territory
            result = condition.CheckNodeValidity(p2Node, player1);
            Assert.AreEqual(TaskValidityState.Ok, result);
            worldManager.SetNodeOwner(p1Node, player1);
        }

        private void CheckWorkerValidity()
        {

        }

        private void CheckWorkersValidity()
        {
            var workers = new List<Worker>();

            //Check Too Few
            var result = condition.CheckWorkersValidity(p1Node, player1, workers);
            Assert.AreEqual(TaskValidityState.TooFewWorkers, result);

            //Check With Valid Workers
            workers.Add(worker);
            result = condition.CheckWorkersValidity(p1Node, player1, workers);
            Assert.AreEqual(TaskValidityState.Ok, result);

            //Check Too Many
            workers.Add(playerManager.AddWorker(player1, workerTypeInfo));
            result = condition.CheckWorkersValidity(p1Node, player1, workers);
            Assert.AreEqual(TaskValidityState.TooManyWorkers, result);
        }
    }
}
