﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Model;

using Buildcraft.Core.NodeTasks.Conditions;
using Buildcraft.ScenarioBuilder;

namespace Buildcraft.Core.Test.ConditionTests
{
    [TestClass]
    public class RepairImprovementConditionTests : BaseTest
    {
        //Model Parts
        private Node node;
        private Player player;
        private Player player2;
        private Worker worker;

        //TypeInfo's
        private ImprovementTypeInfo improvementTypeInfo;
        private WorkerTypeInfo workerTypeInfo;
        private WorkerTypeInfo invalidWorkerTypeInfo;

        //Condition
        private RepairImprovementCondition condition = new RepairImprovementCondition();

        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
            improvementTypeInfo = new ImprovementTypeInfo(ImprovementTypeCodes.City);
            workerTypeInfo = new WorkerTypeInfo(WorkerTypeCodes.Generalist);
            invalidWorkerTypeInfo = new WorkerTypeInfo(WorkerTypeCodes.Administrator);

            condition.MinWorkers = 1;
            condition.MaxWorkers = 1;

            node = worldManager.AddNode();
            player = worldManager.AddPlayer("P1");
            player2 = worldManager.AddPlayer("P2");
            worker = playerManager.AddWorker(player, workerTypeInfo);

            worldManager.SetNodeOwner(node, player);
            worldManager.SetNodeImprovement(node, improvementTypeInfo);
        }

        [TestMethod]
        public void RepairImprovementConditionTests_Basic()
        {
            CheckNodeValidity();
            CheckWorkerValidity();
            CheckWorkersValidity();
        }

        private void CheckNodeValidity()
        {
            //Check if no improvement
            var improvement = node.Improvement;
            node.Improvement = null;
            var result = condition.CheckNodeValidity(node, player);
            Assert.AreEqual(TaskValidityState.NoImprovement, result);
            node.Improvement = improvement;

            //Check if already sabotaged
            node.Improvement.Sabotaged = false;
            result = condition.CheckNodeValidity(node, player);
            Assert.AreEqual(TaskValidityState.ImprovementAlreadyRepaired, result);
            node.Improvement.Sabotaged = true;

            //Check Valid In Friendly territory
            result = condition.CheckNodeValidity(node, player);
            Assert.AreEqual(TaskValidityState.Ok, result);

            //Check in enemy territory
            worldManager.SetNodeOwner(node, player2);
            result = condition.CheckNodeValidity(node, player);
            Assert.AreEqual(TaskValidityState.NotAllowedInEnemyTerritory, result);
        }

        private void CheckWorkerValidity()
        {

        }

        private void CheckWorkersValidity()
        {
            var workers = new List<Worker>();

            //Check Too Few
            var result = condition.CheckWorkersValidity(node, player, workers);
            Assert.AreEqual(TaskValidityState.TooFewWorkers, result);

            //Check With Valid Workers
            workers.Add(worker);
            result = condition.CheckWorkersValidity(node, player, workers);
            Assert.AreEqual(TaskValidityState.Ok, result);

            //Check Too Many
            workers.Add(playerManager.AddWorker(player, workerTypeInfo));
            result = condition.CheckWorkersValidity(node, player, workers);
            Assert.AreEqual(TaskValidityState.TooManyWorkers, result);
        }
    }
}
