﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Model;

using Buildcraft.Core.PlayerTasks;
using Buildcraft.Core.PlayerTasks.Effects;
using Buildcraft.ScenarioBuilder;

namespace Buildcraft.Core.Test.EffectTests
{
    [TestClass]
    public class PlayerEffectTests : BaseTest
    {
        private Player player;

        private ResourceTypeInfo foodResourceType;
        private ResourceTypeInfo moneyResourceType;

        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
            foodResourceType = new ResourceTypeInfo(ResourceTypeCodes.Food);
            moneyResourceType = new ResourceTypeInfo(ResourceTypeCodes.Money);

            player = worldManager.AddPlayer("P1");
        }

        [TestMethod]
        public void PlayerEffectTests_ConsumeResources()
        {
            WorkerTypeInfo generalWorkerType = new WorkerTypeInfo(WorkerTypeCodes.Generalist);
            WorkerTypeInfo farmerWorkerType = new WorkerTypeInfo(WorkerTypeCodes.Farmer);


            generalWorkerType.Consumption[foodResourceType] = 2;
            generalWorkerType.Consumption[moneyResourceType] = 2;
            generalWorkerType.Rank = 1;

            farmerWorkerType.Consumption[foodResourceType] = 2;
            farmerWorkerType.Consumption[moneyResourceType] = 5;
            farmerWorkerType.Rank = 2;

            var generalWorker = playerManager.AddWorker(player, generalWorkerType);
            var farmerWorker = playerManager.AddWorker(player, farmerWorkerType);

            WorkerConsumptionEffect effect = new WorkerConsumptionEffect();
            Container.RegisterInstance<IEffect<PlayerTask>>(effect, "effect");
            Container.Initialize();

            PlayerTaskTypeInfo taskTypeInfo = new PlayerTaskTypeInfo("task", "effect");
            var task = playerManager.AddPlayerTask(player, taskTypeInfo);

            //Fire non effect methods
            effect.Apply(task);
            effect.Refresh(task);
            effect.Remove(task);

            //Fire with sufficient resources
            player.Resources[foodResourceType] = 10;
            player.Resources[moneyResourceType] = 10;

            effect.Fire(task);

            Assert.AreEqual(6, player.Resources[foodResourceType]);
            Assert.AreEqual(3, player.Resources[moneyResourceType]);
            Assert.AreEqual(2, player.AllWorkers.Count);

            //Fire with insufficient resources
            player.Resources[foodResourceType] = 2;
            player.Resources[moneyResourceType] = 5;

            effect.Fire(task);

            Assert.AreEqual(0, player.Resources[foodResourceType]);
            Assert.AreEqual(0, player.Resources[moneyResourceType]);
            Assert.AreEqual(1, player.AllWorkers.Count);
            Assert.AreEqual(farmerWorker, player.AllWorkers[0]);
        }

        [TestMethod]
        public void PlayerEffectTests_WorkerSpawnEffect()
        {
            WorkerTypeInfo typeInfo = new WorkerTypeInfo(WorkerTypeCodes.Generalist);

            WorkerSpawnEffect effect = new WorkerSpawnEffect();
            effect.SpawnType = typeInfo;
            effect.ProductionCost[foodResourceType] = 100;
            effect.ProductionCost[moneyResourceType] = 10;

            PlayerTaskTypeInfo taskTypeInfo = new PlayerTaskTypeInfo("task", "effect");
            var task = playerManager.AddPlayerTask(player, taskTypeInfo);

            Container.RegisterInstance<IEffect<PlayerTask>>(effect, "effect");
            Container.Initialize();

            //Fire non effect methods
            effect.Apply(task);
            effect.Refresh(task);
            effect.Remove(task);

            //Fire with insufficient resources
            player.Resources[foodResourceType] = 0;
            player.Resources[moneyResourceType] = 0;
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                      
            effect.Fire(task);

            Assert.AreEqual(0, player.AllWorkers.Count);

            //Fire with sufficient resources
            player.Resources[foodResourceType] = 100;
            player.Resources[moneyResourceType] = 100;

            effect.Fire(task);

            Assert.AreEqual(1, player.AllWorkers.Count);
            Assert.AreEqual(0, player.Resources[foodResourceType]);
            Assert.AreEqual(90, player.Resources[moneyResourceType]);
        }
    }
}
