﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Model;
using Buildcraft.Core.PlayerActions;
using Buildcraft.Core.Components;

using Buildcraft.Core.NodeTasks.Conditions;
using Rhino.Mocks;
using Buildcraft.ScenarioBuilder;

namespace Buildcraft.Core.Test.ModelActionTests
{
    [TestClass]
    public class TickVictoryConditionsTest : BaseTest
    {
        //Tick Action
        protected TickWorldAction tickWorldAction;

        //World State
        private Player player;
        private Node node;

        private ResourceTypeInfo foodResourceType;
        private ResourceTypeInfo moneyResourceType;

        private ImprovementTypeInfo improvementType;
        private WorkerTypeInfo workerType;


        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
            foodResourceType = new ResourceTypeInfo(ResourceTypeCodes.Food);
            moneyResourceType = new ResourceTypeInfo(ResourceTypeCodes.Money);

            //Set Victory Resource
            var tickAction = Container.RetrieveInstance<TickWorldAction>();
            tickAction.VictoryResourceType = moneyResourceType;

            //Create Improvement Type
            improvementType = Container.AddImprovementTypeInfo(ImprovementTypeCodes.Farm, 1, 5);

            //Create Worker Type
            workerType = Container.AddWorkerTypeInfo(WorkerTypeCodes.Generalist, 0);
            workerType.Consumption[foodResourceType] = 2;
            workerType.Consumption[moneyResourceType] = 2;

            //Create Player And Node
            player = worldManager.AddPlayer("P1");
            node = CreateNode(player, improvementType);

            //Create Tick World Action
            tickWorldAction = Container.RetrieveInstance<TickWorldAction>();
        }

        [TestMethod]
        public void TickWorldTest_TickVictoryConditions()
        {
            //Test no victory at 1 norde, 1000 money
            player.Resources[moneyResourceType] = 1000;
            tickWorldAction.TickWorld(1);
            Assert.AreEqual(GameState.InProgress, world.GameState);
            Assert.AreEqual(null, world.Victor);

            //Add Nodes
            while (world.Nodes.Count < 10)
            {
                var node = worldManager.AddNode();
                worldManager.SetNodeOwner(node, player);
            }

            //Test victory at 10 nodes, 1000 money
            tickWorldAction.TickWorld(1);
            Assert.AreEqual(GameState.GameOver, world.GameState);
            Assert.AreEqual(player, world.Victor);

            //Reset conditions
            world.Victor = null;
            world.GameState = GameState.InProgress;

            //Test no victory at 10 nodes, < 1000 money
            player.Resources[moneyResourceType] -= 5;
            tickWorldAction.TickWorld(1);
            Assert.AreEqual(GameState.InProgress, world.GameState);
            Assert.AreEqual(null, world.Victor);
        }
    }
}
