﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Components;
using Buildcraft.Core.Model;

using Buildcraft.Core.PlayerActions;
using Buildcraft.Core.NodeTasks.Conditions;
using Buildcraft.Core.NodeTasks.Effects;
using Rhino.Mocks;
using Buildcraft.ScenarioBuilder;
using Buildcraft.Core.Interfaces.UnitTasks;


namespace Buildcraft.Core.Test.PlayerActionTests
{
    [TestClass]
    public class PlayerActionsTest : BaseTest
    {
        private Player _player;
        private Node _node;

        private ImprovementTypeInfo _improvementTypeInfo;


        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
            _improvementTypeInfo = new ImprovementTypeInfo(ImprovementTypeCodes.Farm);
            _improvementTypeInfo.MinWorkers = 1;
            _improvementTypeInfo.MaxWorkers = 2;

            _player = CreatePlayer("P1");
            _node = CreateNode(_player, _improvementTypeInfo);
        }


        [TestMethod]
        public void PlayerActionsTest_GetTasksAction()
        {
            //Add First TaskInfo
            ICondition condition = new WorkImprovementCondition();
            IEffect<NodeTask> effect = MockRepository.GenerateMock<IEffect<NodeTask>>();
            NodeTaskTypeInfo taskInfo = new NodeTaskTypeInfo(NodeTaskTypeCodes.WorkImprovement, NodeConditionCodes.WorkImprovementCondition, NodeEffectCodes.WorkImprovementEffect);

            Container.RegisterInstance<ICondition>(condition, NodeConditionCodes.WorkImprovementCondition);
            Container.RegisterInstance<IEffect<NodeTask>>(effect, NodeEffectCodes.WorkImprovementEffect);
            Container.RegisterInstance<NodeTaskTypeInfo>(taskInfo, NodeTaskTypeCodes.WorkImprovement);

            //Add Second TaskInfo
            condition = new SabotageImprovementCondition() { MinWorkers = 1, MaxWorkers = 2 };
            effect = MockRepository.GenerateMock<IEffect<NodeTask>>();
            taskInfo = new NodeTaskTypeInfo(NodeTaskTypeCodes.SabotageImprovement, NodeConditionCodes.SabotageImprovementCondition, NodeEffectCodes.SabotageImprovementEffect);

            Container.RegisterInstance<ICondition>(condition, NodeConditionCodes.SabotageImprovementCondition);
            Container.RegisterInstance<IEffect<NodeTask>>(effect, NodeEffectCodes.SabotageImprovementEffect);
            Container.RegisterInstance<NodeTaskTypeInfo>(taskInfo, NodeTaskTypeCodes.SabotageImprovement);

            Container.Initialize();

            var action = new PlayerActions.PlayerActions(_player, Container);

            var tasks = new List<NodeTaskTypeInfo>();
            action.GetTasksForNode(_node, tasks);

            Assert.AreEqual(1, tasks.Count);
        }

        [TestMethod]
        public void PlayerActionsTest_AddModifyCancelTask()
        {
            PlayerActions.PlayerActions actions = new PlayerActions.PlayerActions(_player, Container);

            var workerType = new WorkerTypeInfo(WorkerTypeCodes.Generalist);
            var condition = Rhino.Mocks.MockRepository.GenerateMock<ICondition>();
            var effect = Rhino.Mocks.MockRepository.GenerateMock<IEffect<NodeTask>>();
            NodeTaskTypeInfo taskInfo = new NodeTaskTypeInfo(NodeTaskTypeCodes.WorkImprovement, "condition", "effect");

            Container.RegisterInstance<WorkerTypeInfo>(workerType, workerType.WorkerTypeCode);
            Container.RegisterInstance<ICondition>(condition, "condition");
            Container.RegisterInstance<IEffect<NodeTask>>(effect, "effect");
            Container.RegisterInstance<NodeTaskTypeInfo>(taskInfo, taskInfo.TaskTypeCode);

            Container.Initialize();

            var worker1 = playerManager.AddWorker(_player, workerType);
            var worker2 = playerManager.AddWorker(_player, workerType);

            //Add Task
            var actionStatus = actions.AddTask(_node, taskInfo, new Worker[] { worker1 });

            Assert.AreEqual(TaskValidityState.Ok, actionStatus);
            Assert.IsTrue(_node.Tasks.Count == 1);
            Assert.IsTrue(worker1.Task == _node.Tasks[0]);
            Assert.AreEqual(2, _player.AllWorkers.Count);
            Assert.AreEqual(1, _player.UnassignedWorkers.Count);


            //Modify Task
            actionStatus = actions.ModifyTask(_node.Tasks[0], new Worker[] { worker1, worker2 });

            Assert.AreEqual(TaskValidityState.Ok, actionStatus);
            Assert.IsTrue(_node.Tasks.Count == 1);
            Assert.IsTrue(worker1.Task == _node.Tasks[0]);
            Assert.IsTrue(worker2.Task == _node.Tasks[0]);
            Assert.AreEqual(2, _player.AllWorkers.Count);
            Assert.AreEqual(0, _player.UnassignedWorkers.Count);


            //Cancel Task
            actionStatus = actions.CancelTask(_node.Tasks[0]);

            Assert.AreEqual(TaskValidityState.Ok, actionStatus);
            Assert.IsTrue(_node.Tasks.Count == 0);
            Assert.IsTrue(worker1.Task == null);
            Assert.IsTrue(worker2.Task == null);
            Assert.AreEqual(2, _player.AllWorkers.Count);
            Assert.AreEqual(2, _player.UnassignedWorkers.Count);
        }



    }
}
