﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Model;

using Buildcraft.Core.PlayerActions;
using Buildcraft.Core.Components;
using Buildcraft.ScenarioBuilder;
using Buildcraft.Core.Interfaces.UnitTasks;

namespace Buildcraft.Core.Test.ScenarioTests
{
    [TestClass]
    public class WorkerMethodScenarioTests : BaseScenarioTest
    {
        //Player Actions
        protected PlayerActions.PlayerActions _playerActions;

        //Nodes
        protected Node _farmNode;

        protected TickWorldAction _tickWorldAction;

        protected override void InitializationComplete()
        {
            _playerActions = new PlayerActions.PlayerActions(player1, Container);

            _tickWorldAction = Container.RetrieveInstance<TickWorldAction>();
        }

        [TestMethod]
        public void WorkerMethodScenarioTests_BasicSinglePlayer()
        {
            //Create a basic scenario containing all the different improvements and some workers.
            CreateBasicScenario();

            //Create Farm Task
            var farmers = player1.UnassignedWorkers.Where(w => w.TypeInfo == farmWorker).ToList();
            var result = _playerActions.AddTask(_farmNode, workImprovementTask, farmers);
            Assert.AreEqual(TaskValidityState.Ok, result);

            //Train our Generalist as a solider
            var generalist = player1.UnassignedWorkers.Where(w => w.TypeInfo == generalWorker).First();
            result = _playerActions.AddTask(p1Capital, trainSoldierTask, new Worker[] { generalist });
            Assert.AreEqual(TaskValidityState.Ok, result);

            //Resources
            player1.Resources[moneyResource] = 100;
            player1.Resources[foodResource] = 0;

            //Tick World for resource production tasks (100)
            for (int i = 0; i < 10; i++)
            {
                tickWorldAction.TickWorld(10);
            }
            
            //Check Worker Trained and task ended
            Assert.AreEqual(soldierWorker, generalist.TypeInfo);
            Assert.AreEqual(0, p1Capital.Tasks.Count);

            //Check we have resources
            Assert.AreEqual(100, player1.Resources[moneyResource]);
            Assert.AreEqual(40, player1.Resources[foodResource]);
            Assert.AreEqual(soldierWorker, generalist.TypeInfo);

            //Tick World for worker consumption task (150)
            for (int i = 0; i < 5; i++)
            {
                tickWorldAction.TickWorld(10);
            }

            //Check Resources Updated
            Assert.AreEqual(81, player1.Resources[moneyResource]);
            Assert.AreEqual(24, player1.Resources[foodResource]);

            var workerCount = player1.AllWorkers.Count;

            //Tick World for second resource production, and worker spawn
            for (int i = 0; i < 5; i++)
            {
                tickWorldAction.TickWorld(10);
            }

            //Check resources
            Assert.AreEqual(81, player1.Resources[moneyResource]);
            Assert.AreEqual(64, player1.Resources[foodResource]);

            //Check worker was produced
            Assert.AreEqual(workerCount + 1, player1.AllWorkers.Count);
        }

        private void CreateBasicScenario()
        {
            //Create Farm
            _farmNode = worldManager.AddNode();
            worldManager.SetNodeImprovement(_farmNode, farmImprovement);
            worldManager.SetNodeOwner(_farmNode, player1);

            //Join Nodes (Improvements in triangle around capital)
            worldManager.AddJoin(p1Capital, _farmNode);

            foreach (var workerType in Container.RetrieveAllInstances<WorkerTypeInfo>())
            {
                for (int i = 0; i < 2; i++)
                {
                    playerManager.AddWorker(player1, workerType);
                }
            }
        }
    }
}
