﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Rhino.Mocks;
using Buildcraft.Core.UnitTasks;
using Buildcraft.Core.Model;
using Buildcraft.Core.TaskProcessors;

namespace Buildcraft.Core.Test.TaskProcessorTests
{
    [TestClass]
    public class CombatProcessorTests : BaseTest
    {
        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
        }

        [TestMethod]
        public void CombatProcessorTests_GeneralTest()
        {
            //Create players
            var defender = CreatePlayer("Defender");
            var attacker = CreatePlayer("Attacker");

            //Create Node
            var node = CreateNode(defender, (ImprovementTypeInfo)null);

            //Setup Combat Buckets
            var nodeDefenceBucket = MockRepository.GenerateStub<IUnitTaskStateBucketNodeDefence>();
            var regionDefenceBucket = MockRepository.GenerateStub<IUnitTaskStateBucketRegionDefence>();
            var nodeAttackBucket = MockRepository.GenerateStub<IUnitTaskStateBucketNodeAttack>();

            //Set bucket owners
            nodeDefenceBucket.Stub(b => b.Owner).Return(defender);
            regionDefenceBucket.Stub(b => b.Owner).Return(defender);
            nodeAttackBucket.Stub(b => b.Owner).Return(attacker);

            //Set Bucket Nodes
            nodeDefenceBucket.Stub(b => b.Node).Return(node);
            regionDefenceBucket.Stub(b => b.Node).Return(node);
            nodeAttackBucket.Stub(b => b.Node).Return(node);

            //Create Combat processor 
            var combatProcessor = new CombatProcessor();
            combatProcessor.Initialize(Container);

            //Register state buckets
            combatProcessor.RegisterStateBucket(nodeDefenceBucket);
            combatProcessor.RegisterStateBucket(regionDefenceBucket);
            combatProcessor.RegisterStateBucket(nodeAttackBucket);

            //Add buckets to node
            node.TaskStateBuckets.Add(nodeDefenceBucket);
            node.TaskStateBuckets.Add(regionDefenceBucket);
            node.TaskStateBuckets.Add(nodeAttackBucket);


            //Process Combat (Defender Wins)
            nodeDefenceBucket.Stub(b => b.Power).Return(1).Repeat.Once();
            regionDefenceBucket.Stub(b => b.Power).Return(1).Repeat.Once();
            nodeAttackBucket.Stub(b => b.Power).Return(2).Repeat.Once();

            //Process
            combatProcessor.Process();
            
            //Assert Defender wins.
            Assert.IsTrue(node.Owner == defender);


            //Process Combat (Attacker Wins)
            nodeDefenceBucket.Stub(b => b.Power).Return(0).Repeat.Once();
            regionDefenceBucket.Stub(b => b.Power).Return(1).Repeat.Once();
            nodeAttackBucket.Stub(b => b.Power).Return(2).Repeat.Once();

            //Process
            combatProcessor.Process();

            //Assert that attacker wins
            Assert.IsTrue(node.Owner == attacker);
        }

        [TestMethod]
        public void CombatProcessorTests_PrimaryAttackerCorrectTest()
        {
            //Create players
            var defender = CreatePlayer("Defender");
            var primaryAttacker = CreatePlayer("Attacker1");
            var secondaryAttacker = CreatePlayer("Attacker2");

            //Create Node
            var node = CreateNode(defender, (ImprovementTypeInfo)null);

            //Setup Combat Buckets
            var nodeDefenceBucket = MockRepository.GenerateStub<IUnitTaskStateBucketNodeDefence>();
            var primaryAttackBucket = MockRepository.GenerateStub<IUnitTaskStateBucketNodeAttack>();
            var secondaryAttackBucket = MockRepository.GenerateStub<IUnitTaskStateBucketNodeAttack>();

            //Set bucket owners
            nodeDefenceBucket.Stub(b => b.Owner).Return(defender);
            primaryAttackBucket.Stub(b => b.Owner).Return(primaryAttacker);
            secondaryAttackBucket.Stub(b => b.Owner).Return(secondaryAttacker);

            //Set Bucket Nodes
            nodeDefenceBucket.Stub(b => b.Node).Return(node);
            primaryAttackBucket.Stub(b => b.Node).Return(node);
            secondaryAttackBucket.Stub(b => b.Node).Return(node);

            //Create Combat processor 
            var combatProcessor = new CombatProcessor();
            combatProcessor.Initialize(Container);

            //Register state buckets
            combatProcessor.RegisterStateBucket(nodeDefenceBucket);
            combatProcessor.RegisterStateBucket(primaryAttackBucket);
            combatProcessor.RegisterStateBucket(secondaryAttackBucket);

            //Add buckets to node
            node.TaskStateBuckets.Add(nodeDefenceBucket);
            node.TaskStateBuckets.Add(primaryAttackBucket);
            node.TaskStateBuckets.Add(secondaryAttackBucket);


            //Process Combat (Secondary Attacker Wins)
            nodeDefenceBucket.Stub(b => b.Power).Return(2);
            primaryAttackBucket.Stub(b => b.Power).Return(1);
            secondaryAttackBucket.Stub(b => b.Power).Return(2);

            //Process
            combatProcessor.Process();

            //Assert that attacker wins
            Assert.IsTrue(node.Owner == secondaryAttacker);
        }
    }
}
