﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.UnitTasks.WorkImprovement;
using Rhino.Mocks;
using Buildcraft.Core.UnitTasks;
using Buildcraft.Core.Model;
using Buildcraft.Core.TaskProcessors;

namespace Buildcraft.Core.Test.TaskProcessorTests
{
    [TestClass]
    public class TaskProductionProcessorTests : BaseTest
    {
        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
        }

        [TestMethod]
        public void TaskProductionProcessorTests_GeneralTest()
        {
            //Resources
            var resource1 = Container.AddResourceTypeInfo("Resource1");
            var resource2 = Container.AddResourceTypeInfo("Resource2");

            //Setup player
            var player = worldManager.AddPlayer("P1");

            //Setup running cost 1
            var production1 = new ResourceValueCollection();
            production1[resource1] = 1;
            production1[resource2] = 0;

            //Setup Running cost 2
            var production2 = new ResourceValueCollection();
            production2[resource1] = 0;
            production2[resource2] = 1;

            //Setup bucket1
            var bucket1 = MockRepository.GenerateStub<IUnitTaskStateBucketProduction>();
            bucket1.Stub(b => b.Owner).Return(player);
            bucket1.Stub(b => b.Production).Return(production1);
            bucket1.Stub(b => b.IsActive).Return(true).Repeat.Once();

            //Setup bucket2
            var bucket2 = MockRepository.GenerateStub<IUnitTaskStateBucketProduction>();
            bucket2.Stub(b => b.Owner).Return(player);
            bucket2.Stub(b => b.Production).Return(production2);
            bucket2.Stub(b => b.IsActive).Return(true).Repeat.Once();

            //Create Processor Register State Bucket
            var processor = new TaskProductionProcessor();
            processor.RegisterStateBucket(bucket1);
            processor.RegisterStateBucket(bucket2);


            //STEP1: Apply production from two enabled tasks (Production)
            processor.Process();

            //Assert Production
            Assert.AreEqual(1, player.Resources[resource1]);
            Assert.AreEqual(1, player.Resources[resource2]);


            //STEP2: Apply production fro two disabled (No production)
            bucket1.Stub(b => b.IsActive).Return(false).Repeat.Once();
            bucket2.Stub(b => b.IsActive).Return(false).Repeat.Once();

            processor.Process();

            //Assert Production
            Assert.AreEqual(1, player.Resources[resource1]);
            Assert.AreEqual(1, player.Resources[resource2]);
        }
    }
}
