﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.UnitTasks.WorkImprovement;
using Rhino.Mocks;
using Buildcraft.Core.UnitTasks;
using Buildcraft.Core.Model;
using Buildcraft.Core.TaskProcessors;

namespace Buildcraft.Core.Test.TaskProcessorTests
{
    [TestClass]
    public class TaskRunningCostProcessorTests : BaseTest
    {

        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
        }

        [TestMethod]
        public void TaskRunningCostProcessorTests_GeneralTest()
        {
            //Resources
            var resource1 = Container.AddResourceTypeInfo("Resource1");
            var resource2 = Container.AddResourceTypeInfo("Resource2");

            //Setup player
            var player = worldManager.AddPlayer("P1");

            //Setup running cost 1
            var runningCost1 = new ResourceValueCollection();
            runningCost1[resource1] = 1;
            runningCost1[resource2] = 0;

            //Setup Running cost 2
            var runningCost2 = new ResourceValueCollection();
            runningCost2[resource1] = 0;
            runningCost2[resource2] = 1;

            //Setup bucket1
            var bucket1 = MockRepository.GenerateStub<IUnitTaskStateBucketRunningCost>();
            bucket1.Stub(b => b.Owner).Return(player);
            bucket1.Stub(b => b.RunningCost).Return(runningCost1);
            bucket1.IsPaidFor = true;

            //Setup bucket2
            var bucket2 = MockRepository.GenerateStub<IUnitTaskStateBucketRunningCost>();
            bucket2.Stub(b => b.Owner).Return(player);
            bucket2.Stub(b => b.RunningCost).Return(runningCost2);
            bucket2.IsPaidFor = true;

            //Create and setup processor
            var processor = new TaskRunningCostProcessor();
            processor.RegisterStateBucket(bucket1);
            processor.RegisterStateBucket(bucket2);


            //STEP 1: Process with both tasks unaffordable
            processor.Process();

            Assert.IsFalse(bucket1.IsPaidFor);
            Assert.IsFalse(bucket2.IsPaidFor);


            //STEP 2: Process with bucket1 Affordable
            player.Resources[resource1] = 1;
            processor.Process();

            Assert.IsTrue(bucket1.IsPaidFor);
            Assert.IsFalse(bucket2.IsPaidFor);
            Assert.AreEqual(0, player.Resources[resource1]);
            

            //STEP 3: Process with bucket2 Affordable
            player.Resources[resource2] = 1;
            processor.Process();

            Assert.IsFalse(bucket1.IsPaidFor);
            Assert.IsTrue(bucket2.IsPaidFor);
            Assert.AreEqual(0, player.Resources[resource2]);
        }

    }
}
