﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.TaskProcessors;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Model;
using Buildcraft.Core.Components;

namespace Buildcraft.Core.Test.TaskProcessorTests
{
    [TestClass]
    public class UnitConsumptionProcessorTests : BaseTest
    {

        protected override void CustomInitialize()
        {
        }

        protected override void ConfigureWorld()
        {
        }

        protected override Interfaces.ITaskScheduler TaskSchedulerMock
        {
            get { return new BasicTaskScheduler(1); }
        }

        [TestMethod]
        public void UnitConsumptionProcessorTests_GeneralTest()
        {
            //Create test resource
            var resource = new ResourceTypeInfo("Resource");

            //Setup Unit Type Info
            var unitTypeInfo = new UnitTypeInfo("Worker");
            unitTypeInfo.Consumption[resource] = 1;

            //Setup player
            var player = CreatePlayer("Player1");
            player.Resources[resource] = 2;

            //Add Units
            var unit1 = playerManager.AddUnit(player, unitTypeInfo);
            var unit2 = playerManager.AddUnit(player, unitTypeInfo);

            //Create and setup processor
            var processor = new UnitConsumptionProcessor();
            processor.Initialize(Container);


            //STEP 1: Process units - Adequate supply
            processor.Process();

            Assert.IsTrue(unit1.IsSupplied);
            Assert.IsTrue(unit2.IsSupplied);
            Assert.AreEqual(0, player.Resources[resource]);


            //STEP 2: Process units - No Supply
            processor.Process();

            Assert.IsFalse(unit1.IsSupplied);
            Assert.IsFalse(unit2.IsSupplied);
            Assert.AreEqual(0, player.Resources[resource]);


            //STEP 3: Process units - Partial Supply
            player.Resources[resource] = 1;
            processor.Process();

            Assert.IsTrue(unit1.IsSupplied);
            Assert.IsFalse(unit2.IsSupplied);
            Assert.AreEqual(0, player.Resources[resource]);
        }
    }
}
