﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Practices.Unity;

namespace Buildcraft.Core.Test
{
    /// <summary>
    /// Test container for implementing dependency injection in the game enviroment.
    /// </summary>
    public class TestContainer : IContainer
    {
        private UnityContainer _container = new UnityContainer();
        private List<IInitializable> _initializables = new List<IInitializable>();


        #region IContainer Members

        public void RegisterInstance<T>(T instance)
        {
            _container.RegisterInstance<T>(instance);

            if (instance is IInitializable)
                _initializables.Add((IInitializable)instance);
        }

        public void RegisterInstance<T>(T instance, string objectCode)
        {
            _container.RegisterInstance<T>(objectCode, instance);

            if (instance is IInitializable)
                _initializables.Add((IInitializable)instance);
        }


        public T RetrieveInstance<T>()
        {
            var instance = _container.Resolve<T>();

            if (instance == null)
                throw new ApplicationException("Type Not Found: " + typeof(T).Name);

            return instance;
        }

        public T RetrieveInstance<T>(string objectCode)
        {
            var instance = _container.Resolve<T>(objectCode);

            if (instance == null)
                throw new ApplicationException(string.Format("Type {0} not found with code {1} ", typeof(T).Name, objectCode));

            return instance;
        }

        public List<T> RetrieveAllInstances<T>()
        {
            return _container.ResolveAll<T>().ToList();
        }

        public void Initialize()
        {
            foreach (var initializables in _initializables)
            {
                initializables.Initialize(this);
            }
        }

        #endregion
    }
}
