﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.Model;
using Buildcraft.Core.UnitTasks.AttackNode;

namespace Buildcraft.Core.Test.UnitTaskTests
{
    /// <summary>
    /// Summary description for AttackNodeTaskInfoTests
    /// </summary>
    [TestClass]
    public class AttackNodeTaskTests : BaseScenarioTest
    {
        protected override void InitializationComplete()
        {
        }

        [TestMethod]
        public void AttackNodeTaskInfo_CreateBucketWithDoubleState()
        {
            //Create Parts
            var player = CreatePlayer("P1");
            var unitTypeInfo = new UnitTypeInfo("type");
            var unit = playerManager.AddUnit(player, unitTypeInfo);
            var node = worldManager.AddNode();

            //Create item under test
            var taskInfo = new AttackNodeTaskTypeInfo();


            //NOTE: Not happy with this solution - Make better bootstrapping process.

            //Setup Combat Mapping
            var combatMapping = new Dictionary<UnitTypeInfo, int>();
            combatMapping.Add(unitTypeInfo, 1);

            //Setup Consumption Mapping
            var consumptionMapping = new Dictionary<UnitTypeInfo, ResourceValueCollection>();
            consumptionMapping.Add(unitTypeInfo, new ResourceValueCollection());
            consumptionMapping[unitTypeInfo][moneyResource] = 2;

            //Load into task
            taskInfo.Setup(combatMapping, consumptionMapping);


            //Create first State from the task info
            var state1 = (AttackNodeTaskState)taskInfo.CreateTaskState(node, unit);
            Assert.IsNotNull(state1);

            //Assert State Setup correctly
            Assert.AreEqual(1, state1.RunningCost.Count());
            Assert.AreEqual(2, state1.RunningCost[moneyResource]);

            Assert.AreEqual(1, state1.Power);


            //Create Second State
            var state2 = (AttackNodeTaskState)taskInfo.CreateTaskState(node, unit);
            Assert.IsNotNull(state2);

            //Assert State Setup correctly
            Assert.AreEqual(1, state2.RunningCost.Count());
            Assert.AreEqual(2, state2.RunningCost[moneyResource]);

            Assert.AreEqual(1, state1.Power);


            //Create a Bucket from the task info
            var bucket = (AttackNodeTaskStateBucket)taskInfo.CreateTaskStateBucket(node, unit.Owner);
            Assert.IsNotNull(bucket);

            //Add Task States
            bucket.AddTaskState(state1, false); //No recalc
            bucket.AddTaskState(state2, false); //No recalc
            bucket.Recalculate(false); //Top level recalc only


            //Assert Bucket was setup correctly.
            Assert.AreEqual(1, bucket.RunningCost.Count());
            Assert.AreEqual(4, bucket.RunningCost[moneyResource]);

            Assert.AreEqual(2, bucket.Power);
        }
    }
}
