﻿using System;
using System.Text;
using System.Collections.Generic;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Buildcraft.Core.UnitTasks.WorkImprovement;
using Buildcraft.Core.Model;

namespace Buildcraft.Core.Test.UnitTaskTests
{
    [TestClass]
    public class WorkImprovementTaskTests : BaseScenarioTest
    {
        protected override void InitializationComplete()
        {
        }

        [TestMethod]
        public void WorkImprovementTaskInfoTests_CreateBucketWithSingleState()
        {
            //Create Parts
            var player = CreatePlayer("P1");
            var unitTypeInfo = new UnitTypeInfo("type");
            var unit = playerManager.AddUnit(player, unitTypeInfo);
            var node = worldManager.AddNode();

            //Create item under test
            var taskInfo = new WorkImprovementTaskTypeInfo();


            //NOTE: Not happy with this solution - Messy, and could be cleaner.

            //Setup Production Mapping
            var productionMapping = new Dictionary<UnitTypeInfo, ResourceValueCollection>();
            productionMapping.Add(unitTypeInfo, new ResourceValueCollection());
            productionMapping[unitTypeInfo][moneyResource] = 2;

            //Setup Consumption Mapping
            var consumptionMapping = new Dictionary<UnitTypeInfo, ResourceValueCollection>();
            consumptionMapping.Add(unitTypeInfo, new ResourceValueCollection());
            consumptionMapping[unitTypeInfo][moneyResource] = 2;

            //Load into task
            taskInfo.Setup(productionMapping, consumptionMapping);


            //Create a State from the task info
            var state = (WorkImprovementTaskState)taskInfo.CreateTaskState(node, unit);
            Assert.IsNotNull(state);

            //Assert State Setup correctly
            Assert.AreEqual(1, state.RunningCost.Count());
            Assert.AreEqual(2, state.RunningCost[moneyResource]);

            Assert.AreEqual(1, state.Production.Count());
            Assert.AreEqual(2, state.Production[moneyResource]);


            //Create a Bucket from the task info
            var bucket = (WorkImprovementTaskStateBucket)taskInfo.CreateTaskStateBucket(node, unit.Owner);
            Assert.IsNotNull(bucket);

            //Add Task State (Recalc is automatic)
            bucket.AddTaskState(state, true);

            //Assert Bucket was setup correctly.
            Assert.AreEqual(1, bucket.RunningCost.Count());
            Assert.AreEqual(2, bucket.RunningCost[moneyResource]);

            Assert.AreEqual(1, bucket.Production.Count());
            Assert.AreEqual(2, bucket.Production[moneyResource]);
        }


        [TestMethod]
        public void WorkImprovementTaskInfoTests_CreateBucketWithDoubleState()
        {
            //Create Parts
            var player = CreatePlayer("P1");
            var unitTypeInfo = new UnitTypeInfo("type");
            var unit = playerManager.AddUnit(player, unitTypeInfo);
            var node = worldManager.AddNode();

            //Create item under test
            var taskInfo = new WorkImprovementTaskTypeInfo();


            //NOTE: Not happy with this solution - Make better bootstrapping process.

            //Setup Production Mapping
            var productionMapping = new Dictionary<UnitTypeInfo, ResourceValueCollection>();
            productionMapping.Add(unitTypeInfo, new ResourceValueCollection());
            productionMapping[unitTypeInfo][moneyResource] = 2;

            //Setup Consumption Mapping
            var consumptionMapping = new Dictionary<UnitTypeInfo, ResourceValueCollection>();
            consumptionMapping.Add(unitTypeInfo, new ResourceValueCollection());
            consumptionMapping[unitTypeInfo][moneyResource] = 2;

            //Load into task
            taskInfo.Setup(productionMapping, consumptionMapping);


            //Create first State from the task info
            var state1 = (WorkImprovementTaskState)taskInfo.CreateTaskState(node, unit);
            Assert.IsNotNull(state1);

            //Assert State Setup correctly
            Assert.AreEqual(1, state1.RunningCost.Count());
            Assert.AreEqual(2, state1.RunningCost[moneyResource]);

            Assert.AreEqual(1, state1.Production.Count());
            Assert.AreEqual(2, state1.Production[moneyResource]);


            //Create Second State
            var state2 = (WorkImprovementTaskState)taskInfo.CreateTaskState(node, unit);
            Assert.IsNotNull(state2);

            //Assert State Setup correctly
            Assert.AreEqual(1, state2.RunningCost.Count());
            Assert.AreEqual(2, state2.RunningCost[moneyResource]);

            Assert.AreEqual(1, state2.Production.Count());
            Assert.AreEqual(2, state2.Production[moneyResource]);


            //Create a Bucket from the task info
            var bucket = (WorkImprovementTaskStateBucket)taskInfo.CreateTaskStateBucket(node, unit.Owner);
            Assert.IsNotNull(bucket);

            //Add Task States (Recalc is automatic)
            bucket.AddTaskState(state1, false); //No recalc
            bucket.AddTaskState(state2, false); //No recalc
            bucket.Recalculate(false); //Top level recalc only


            //Assert Bucket was setup correctly.
            Assert.AreEqual(1, bucket.RunningCost.Count());
            Assert.AreEqual(4, bucket.RunningCost[moneyResource]);

            Assert.AreEqual(1, bucket.Production.Count());
            Assert.AreEqual(4, bucket.Production[moneyResource]);
        }
    }
}
