﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Model;
using Buildcraft.Core.Interfaces.UnitTasks.TaskInfo;
using Buildcraft.Core.UnitTasks;

namespace Buildcraft.Core.Components
{
    public static class ConstraintValidator
    {
        public static bool Validate(Node node, Unit unit, IUnitTaskInfo unitTask)
        {
            //Unit Type Constraint
            if (unitTask is IUnitTaskUnitTypeConstraint &&
                !Validate(unit, (IUnitTaskUnitTypeConstraint)unitTask))
                return false;

            //Test Improvement Type Constraint
            if (unitTask is IUnitTaskImprovementTypeConstraint &&
                !Validate(node, (IUnitTaskImprovementTypeConstraint)unitTask))
                return false;
            
            //Test Node Owner Constraint
            if (unitTask is IUnitTaskNodeOwnerConstraint &&
                !Validate(node, unit, (IUnitTaskNodeOwnerConstraint)unitTask))
                return false;

            //Test Node Effect Constraints
            if (unitTask is IUnitTaskNodeEffectTypeConstraint &&
                !Validate(node, (IUnitTaskNodeEffectTypeConstraint)unitTask))
                return false;

            return true;
        }

        public static bool Validate(Unit unit, IUnitTaskUnitTypeConstraint constraint)
        {
            return constraint.UnitTypeCheck.IsValid(unit);
        }

        public static bool Validate(Node node, IUnitTaskImprovementTypeConstraint constraint)
        {
            return constraint.ImprovementTypeCheck.IsValid(node.Improvement);
        }

        public static bool Validate(Node node, Unit unit, IUnitTaskNodeOwnerConstraint constraint)
        {
            if (constraint.Owner == OwnerConstraint.Self &&
                node.Owner != unit.Owner)
                return false;

            if (constraint.Owner == OwnerConstraint.Enemy &&
                node.Owner == unit.Owner)
                return false;

            return true;
        }

        public static bool Validate(Node node, IUnitTaskNodeEffectTypeConstraint constraint)
        {
            throw new NotImplementedException();
        }
    }
}
