﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.Components
{
    public class InclusiveSetCheck<T> : IEntityTypeCheck<T>
        where T : ITypeInfo
    {
        private List<T> _types = new List<T>();

        public InclusiveSetCheck(IEnumerable<T> typeCollection)
        {
            if (typeCollection == null)
                throw new ArgumentNullException();

            _types.AddRange(typeCollection);
        }

        #region IEntityTypeCheck<T> Members

        public bool IsValid(ITypedEntity<T> entity)
        {
            var info = entity.TypeInfo;
            return _types.Contains(info);
        }

        #endregion
    }

    public class ExclusiveSetCheck<T> : IEntityTypeCheck<T>
        where T : ITypeInfo
    {
        private List<T> _types = new List<T>();

        public ExclusiveSetCheck(IEnumerable<T> typeCollection)
        {
            if (typeCollection == null)
                throw new ArgumentNullException();

            _types.AddRange(typeCollection);
        }

        #region IEntityTypeCheck<T> Members

        public bool IsValid(ITypedEntity<T> entity)
        {
            var info = entity.TypeInfo;
            return !_types.Contains(info);
        }

        #endregion
    }

    public class SingleTypeSetCheck<T> : IEntityTypeCheck<T>
        where T : ITypeInfo
    {
        private T _type;

        public SingleTypeSetCheck(T type)
        {
            if (type == null)
                throw new ArgumentNullException();

            _type = type;
        }

        #region IEntityTypeCheck<T> Members

        public bool IsValid(ITypedEntity<T> entity)
        {
            return _type.Equals(entity.TypeInfo);
        }

        #endregion
    }
}
