﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

namespace Buildcraft.Core.Model
{
    /// <summary>
    /// Interface for defining an Effect. An effect is a component of a Task,
    /// where the task will fire it's associated effect(s) at certain points.
    /// </summary>
    public interface IEffect<T>
    {
        /// <summary>
        /// Apply the effect when the task is added.
        /// </summary>
        /// <param name="task">The task that has been added</param>
        void Apply(T task);

        /// <summary>
        /// Fire the effect when the task is completed.
        /// </summary>
        /// <param name="task">The task to fire for.</param>
        void Fire(T task);

        /// <summary>
        /// Recalculate the state of the effect
        /// </summary>
        /// <param name="task">The task to recalculate for.</param>
        void Refresh(T task);

        /// <summary>
        /// Remove the effect when the task is removed.
        /// </summary>
        /// <param name="task"></param>
        void Remove(T task);
    }
}
