﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.UnitTasks;

namespace Buildcraft.Core.Interfaces
{
    /// <summary>
    /// Core interface for the task scheduler.
    /// </summary>
    public interface ITaskScheduler
    {
        /// <summary>
        /// Register a task processor with the scheduler.
        /// </summary>
        /// <param name="processor"></param>
        void RegisterTaskProcessor(ITaskProcessor processor);

        /// <summary>
        /// Unregister a task processor from the scheduler.
        /// </summary>
        /// <param name="processor"></param>
        void UnregisterTaskProcessor(ITaskProcessor processor);


        /// <summary>
        /// Register a new task state bucket with the task scheduler.
        /// </summary>
        /// <param name="bucket"></param>
        void RegisterStateBucket(IUnitTaskStateBucket bucket);

        /// <summary>
        /// Unregister an existing task state bucket with the task scheduler.
        /// </summary>
        /// <param name="bucket"></param>
        void UnregisterStateBucket(IUnitTaskStateBucket bucket);


        /// <summary>
        /// Register a new task state with the task scheduler.
        /// </summary>
        /// <param name="state"></param>
        void RegisterState(IUnitTaskStateItem state);

        /// <summary>
        /// Unregister an existing task state with the task scheduler.
        /// </summary>
        /// <param name="state"></param>
        void UnregisterState(IUnitTaskStateItem state);


        /// <summary>
        /// Tick the task scheduler by the specified number of ticks. This should fire off all the relevant processes subscribed to this scheduler.
        /// </summary>
        /// <param name="ticks"></param>
        void Tick(int ticks);
    }
}
