﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.NodeTasks.Effects;
using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.Model
{
    /// <summary>
    /// Collection for effect values.
    /// </summary>
    public class EffectValueCollection
    {
        private List<EffectValueItem> _values = new List<EffectValueItem>();


        public void Add(int taskId, IEffect<Model.NodeTask> effect, int value)
        {
            var existing = Find(taskId, effect);

            if (existing != null)
                throw new BuildcraftException("Effect value already added");

            _values.Add(new EffectValueItem() 
            { 
                TaskId = taskId, 
                Effect = effect, 
                Value = value 
            });
        }

        public void Modify(int taskId, IEffect<Model.NodeTask> effect, int value)
        {
            var existing = Find(taskId, effect);

            if (existing == null)
                throw new BuildcraftException("Does not exist.");

            existing.Value = value;
        }

        public void Remove(int taskId, IEffect<Model.NodeTask> effect)
        {
            var existing = Find(taskId, effect);

            if (existing == null)
                throw new BuildcraftException("Does not exist.");

            _values.Remove(existing);
        }

        public int GetTotalValueForEffect(IEffect<Model.NodeTask> effect)
        {
            return _values
                    .Where(v => v.Effect == effect)
                    .Sum(v => v.Value);
        }

        private EffectValueItem Find(int taskId, IEffect<Model.NodeTask> effect)
        {
            return _values.Where(value =>
                    value.TaskId == taskId &&
                    value.Effect == effect)
                .SingleOrDefault();
        }

    }

    public class EffectValueItem
    {
        public int TaskId;
        public IEffect<Model.NodeTask> Effect;
        public int Value;
    }
}
