﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.Model
{
    /// <summary>
    /// An improvement type.
    /// </summary>
    public class ImprovementTypeInfo : ITypeInfo
    {
        /// <summary>
        /// Type code
        /// </summary>
        public string ImprovementTypeCode;

        /// <summary>
        /// Does the improvement type have a worker restriction
        /// </summary>
        public WorkerTypeInfo WorkerTypeRestriction;

        /// <summary>
        /// Maximum number of workers who can work this improvement
        /// </summary>
        public int MaxWorkers;

        /// <summary>
        /// Minimum number of workers who can work this improvement
        /// </summary>
        public int MinWorkers;


        /// <summary>
        /// The worker type that produces double resources
        /// </summary>
        public WorkerTypeInfo SpecialistWorkerType;

        /// <summary>
        /// The resource this node produces.
        /// </summary>
        public ResourceTypeInfo ResourceProductionType;

        /// <summary>
        /// The amount of the resource generated by this node, per tick.
        /// </summary>
        public int ResourceProductionRate;


        public ImprovementTypeInfo(string improvementTypeCode)
        {
            ImprovementTypeCode = improvementTypeCode;
        }

        public override string ToString()
        {
            return ImprovementTypeCode.ToString();
        }

        #region ITypeInfo Members

        public string TypeCode
        {
            get { return ImprovementTypeCode; }
        }

        #endregion
    }
}
