﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;


namespace Buildcraft.Core.Model
{
    public class ResourceValueCollection : IEnumerable<ResourceTypeInfo>
    {
        private Dictionary<ResourceTypeInfo, int> ResourceDictionary = new Dictionary<ResourceTypeInfo, int>();

        /// <summary>
        /// Get or set the quantity of a resource type.
        /// </summary>
        /// <param name="resourceType">The resource type to use.</param>
        /// <returns>The current value - 0 if the key does not exist.</returns>
        public int this[ResourceTypeInfo resourceTypeCode]
        {
            get
            {
                if (!ResourceDictionary.ContainsKey(resourceTypeCode))
                    return 0;

                return ResourceDictionary[resourceTypeCode];
            }
            set
            {
                ResourceDictionary[resourceTypeCode] = value;
            }
        }

        /// <summary>
        /// Add the value of another resource collection to this one.
        /// </summary>
        /// <param name="collection"></param>
        public void Add(ResourceValueCollection addCollection)
        {
            foreach (var rv in addCollection)
            {
                this[rv] += addCollection[rv];
            }
        }

        /// <summary>
        /// Subtract the value of another resource collection from this one.
        /// </summary>
        /// <param name="subtractCollection"></param>
        public bool CanSubtract(ResourceValueCollection subtractCollection)
        {
            foreach (var rv in subtractCollection)
            {
                if (this[rv] < subtractCollection[rv])
                    return false;
            }

            return true;
        }

        /// <summary>
        /// Subtract the value of another resource collection from this one.
        /// </summary>
        /// <param name="subtractCollection"></param>
        public void Subtract(ResourceValueCollection subtractCollection)
        {
            foreach (var rv in subtractCollection)
            {
                this[rv] -= subtractCollection[rv];
            }
        }

        /// <summary>
        /// Clear the resource value collection
        /// </summary>
        public void Clear()
        {
            ResourceDictionary.Clear();
        }

        #region IEnumerable<ResourceType> Members

        public IEnumerator<ResourceTypeInfo> GetEnumerator()
        {
            return ResourceDictionary.Keys.GetEnumerator();
        }

        #endregion

        #region IEnumerable Members

        System.Collections.IEnumerator System.Collections.IEnumerable.GetEnumerator()
        {
            return ResourceDictionary.Keys.GetEnumerator();
        }

        #endregion
    }
}
