﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.NodeTasks.Conditions
{
    public class AttackNodeCondition : ICondition
    {
        //Worker Restrictions
        public int MaxWorkers;
        public int MinWorkers;

        public TaskParameterType ParameterType 
        {
            get { return TaskParameterType.None; }
        }

        public TaskValidityState CheckNodeValidity(Node node, Player player)
        {
            //Check can perform in friendly territory
            if (node.Owner == player)
                return TaskValidityState.NotAllowedInFriendlyTerritory;

            //Check Borders friendly territory
            bool borderNode = false;
            foreach (var join in node.Joins)
            {
                if (join.ToNode.Owner == player)
                {
                    borderNode = true;
                    break;
                }
            }

            if (!borderNode)
                return TaskValidityState.MustBeAdjacentToBorder;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkerValidity(Node node, Player player, Worker worker)
        {
            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkersValidity(Node node, Player player, IEnumerable<Worker> workers)
        {
            int count = workers.Count();

            if (count < MinWorkers)
                return TaskValidityState.TooFewWorkers;

            if (count > MaxWorkers)
                return TaskValidityState.TooManyWorkers;

            //Valid for workers check
            foreach (var worker in workers)
            {
                if (CheckWorkerValidity(node, player, worker) != TaskValidityState.Ok)
                    return TaskValidityState.OneOrMoreWorkerIsInvalid;
            }

            return TaskValidityState.Ok;
        }
    }
}
