﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Buildcraft.Core.Model;

namespace Buildcraft.Core.NodeTasks.Conditions
{
    public class PatrolBorderCondition : ICondition
    {
        public int MinWorkers;
        public int MaxWorkers;
        public ImprovementTypeInfo ImprovementRestriction;


        public TaskParameterType ParameterType
        {
            get { return TaskParameterType.None; }
        }

        public TaskValidityState CheckNodeValidity(Node node, Player player)
        {
            //Check has improvement
            if (node.Improvement == null)
                return TaskValidityState.NoImprovement;

            //Check has correct improvement
            if (node.Improvement.TypeInfo != ImprovementRestriction)
                return TaskValidityState.WrongImprovement;

            //Check is friendly territory
            if (node.Owner != player)
                return TaskValidityState.NotAllowedInEnemyTerritory;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkerValidity(Node node, Player player, Worker worker)
        {
            if (CheckNodeValidity(node, player) != TaskValidityState.Ok)
                return TaskValidityState.InvalidForNode;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkersValidity(Node node, Player player, IEnumerable<Worker> workers)
        {
            if (CheckNodeValidity(node, player) != TaskValidityState.Ok)
                return TaskValidityState.InvalidForNode;

            int count = 0;

            //Valid for workers check
            foreach (var worker in workers)
            {
                if (CheckWorkerValidity(node, player, worker) != TaskValidityState.Ok)
                    return TaskValidityState.OneOrMoreWorkerIsInvalid;

                count++;
            }

            if (count < MinWorkers)
                return TaskValidityState.TooFewWorkers;

            if (count > MaxWorkers)
                return TaskValidityState.TooManyWorkers;

            return TaskValidityState.Ok;
        }

    }
}
