﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.NodeTasks.Conditions
{
    public class RepairImprovementCondition : ICondition
    {
        public int MaxWorkers;
        public int MinWorkers;

        public TaskParameterType ParameterType 
        {
            get { return TaskParameterType.None; }
        }

        public TaskValidityState CheckNodeValidity(Node node, Player player)
        {
            //No improvement
            if (node.Improvement == null)
                return TaskValidityState.NoImprovement;

            //Already repaired
            if (!node.Improvement.Sabotaged)
                return TaskValidityState.ImprovementAlreadyRepaired;

            //Check can perform in enemy territory
            if (node.Owner != player)
                return TaskValidityState.NotAllowedInEnemyTerritory;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkerValidity(Node node, Player player, Worker worker)
        {
            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkersValidity(Node node, Player player, IEnumerable<Worker> workers)
        {
            int count = 0;

            //Valid for workers check
            foreach (var worker in workers)
            {
                if (CheckWorkerValidity(node, player, worker) != TaskValidityState.Ok)
                    return TaskValidityState.OneOrMoreWorkerIsInvalid;

                count++;
            }

            if (count < MinWorkers)
                return TaskValidityState.TooFewWorkers;

            if (count > MaxWorkers)
                return TaskValidityState.TooManyWorkers;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckParameter(Node node, Player player, Worker worker, object parameter)
        {
            if (parameter != null)
                return TaskValidityState.InvalidParameter;

            return TaskValidityState.Ok;
        }
    }
}
