﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.NodeTasks.Conditions
{
    public class SabotageImprovementCondition : ICondition
    {
        public int MaxWorkers;
        public int MinWorkers;

        public TaskParameterType ParameterType 
        {
            get { return TaskParameterType.None; }
        }

        public TaskValidityState CheckNodeValidity(Node node, Player player)
        {
            //No improvement
            if (node.Improvement == null)
                return TaskValidityState.NoImprovement;

            //Already sabotaged
            if (node.Improvement.Sabotaged)
                return TaskValidityState.ImprovementAlreadySabotaged;

            //Check can perform in enemy territory
            if (node.Owner == player)
                return TaskValidityState.NotAllowedInFriendlyTerritory;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkerValidity(Node node, Player player, Worker worker)
        {
            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkersValidity(Node node, Player player, IEnumerable<Worker> workers)
        {
            int count = workers.Count();

            if (count < MinWorkers)
                return TaskValidityState.TooFewWorkers;

            if (count > MaxWorkers)
                return TaskValidityState.TooManyWorkers;

            //Valid for workers check
            foreach (var worker in workers)
            {
                if (CheckWorkerValidity(node, player, worker) != TaskValidityState.Ok)
                    return TaskValidityState.OneOrMoreWorkerIsInvalid;
            }

            return TaskValidityState.Ok;
        }

    }
}
