﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.NodeTasks.Conditions
{
    public class TrainWorkerCondition : ICondition
    {
        /// <summary>
        /// The required improvement for this condition
        /// </summary>
        public ImprovementTypeInfo ImprovementRestriction;

        /// <summary>
        /// The worker type the worker will be trained into. The condition will reject a worker of this type.
        /// </summary>
        public WorkerTypeInfo TargetWorkerType;

        public int MaxWorkers;
        public int MinWorkers;

        public TaskParameterType ParameterType 
        {
            get { return TaskParameterType.WorkerType; }
        }

        public TaskValidityState CheckNodeValidity(Node node, Player player)
        {
            //Check can perform in enemy territory
            if (node.Owner != player)
                return TaskValidityState.NotAllowedInEnemyTerritory;

            if (node.Improvement == null)
                return TaskValidityState.NoImprovement;

            if (ImprovementRestriction != null && node.Improvement.TypeInfo != ImprovementRestriction)
                return TaskValidityState.WrongImprovement;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkerValidity(Node node, Player player, Worker worker)
        {
            if (worker.TypeInfo == TargetWorkerType)
                return TaskValidityState.InvalidForWorker;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkersValidity(Node node, Player player, IEnumerable<Worker> workers)
        {
            int count = workers.Count();

            //Quantity Check
            if (count < MinWorkers)
                return TaskValidityState.TooFewWorkers;

            if (count > MaxWorkers)
                return TaskValidityState.TooManyWorkers;

            //Worker Validity Check
            foreach (var worker in workers)
            {
                if (CheckWorkerValidity(node, player, worker) != TaskValidityState.Ok)
                    return TaskValidityState.OneOrMoreWorkerIsInvalid;
            }

            return TaskValidityState.Ok;
        }

    }
}
