﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;

using Buildcraft.Core.Model;
using System.Diagnostics;

namespace Buildcraft.Core.NodeTasks.Conditions
{
    public class WorkImprovementCondition : ICondition
    {
        public TaskParameterType ParameterType
        {
            get { return TaskParameterType.None; }
        }

        public TaskValidityState CheckNodeValidity(Node node, Player player)
        {
            //Check has improvement
            if (node.Improvement == null)
                return TaskValidityState.NoImprovement;

            //Check is friendly territory
            if (node.Owner != player)
                return TaskValidityState.NotAllowedInEnemyTerritory;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkerValidity(Node node, Player player, Worker worker)
        {

            if (worker == null)
                throw new ArgumentNullException("Worker");

            if (CheckNodeValidity(node, player) != TaskValidityState.Ok)
                return TaskValidityState.InvalidForNode;

            var improvement = node.Improvement;
            var improvementTypeInfo = improvement.TypeInfo;

            //Worker Type Check
            if (improvementTypeInfo.WorkerTypeRestriction != null && worker.TypeInfo != improvementTypeInfo.WorkerTypeRestriction)
                return TaskValidityState.WorkerIsWrongType;

            return TaskValidityState.Ok;
        }

        public TaskValidityState CheckWorkersValidity(Node node, Player player, IEnumerable<Worker> workers)
        {
            var result = CheckNodeValidity(node, player);
            if (result != TaskValidityState.Ok)
                return result;

            //Unpack Improvement
            var improvement = node.Improvement;
            var improvementTypeInfo = improvement.TypeInfo;

            //Check Conditions
            var count = workers.Count();

            if (count < improvementTypeInfo.MinWorkers)
                return TaskValidityState.TooFewWorkers;

            if (count > improvementTypeInfo.MaxWorkers)
                return TaskValidityState.TooManyWorkers;

            //Valid for workers check
            foreach (var worker in workers)
            {

                if (CheckWorkerValidity(node, player, worker) != TaskValidityState.Ok)
                    return TaskValidityState.OneOrMoreWorkerIsInvalid;
            }

            return TaskValidityState.Ok;
        }

    }
}
