﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Components;
using Buildcraft.Core.Model;

using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.NodeTasks.Effects
{
    public class AttackNodeEffect : IEffect<Model.NodeTask>, IInitializable
    {
        /// <summary>
        /// Resources produced when fired, per worker
        /// </summary>
        public int AttackPerWorker;

        /// <summary>
        /// The specialist type that will provide a multiplier.
        /// </summary>
        public WorkerTypeInfo SpecialistType;

        /// <summary>
        /// The multiplier the specialist provides
        /// </summary>
        public float SpecialistMultiplier;


        private WorldManager _worldManager;
        private IHostNotify _hostNotify;

        public void Initialize(IBuildcraftContainer container)
        {
            _worldManager = container.RetrieveInstance<WorldManager>();
            _hostNotify = container.RetrieveInstance<IHostNotify>();
        }

        #region IEffect Members

        public void Apply(NodeTask task)
        {
        }

        public void Fire(NodeTask task)
        {
            //Get combat values
            var attack = CalculateAttack(task);
            var defence = 0;
            
            if (task.Node.Owner != null)
                defence += task.Node.Owner.Defence;

            if (attack > defence)
            {
                _worldManager.SetNodeOwner(task.Node, task.Owner);
                
                //TODO: Notify of node conquest
            }
            else
            {
                //TODO: Notify node defended
            }
        }

        public void Refresh(NodeTask task)
        {
            throw new NotImplementedException();
        }

        public void Remove(NodeTask task)
        {
        }

        #endregion


        private int CalculateAttack(NodeTask task)
        {
            int total = 0;

            foreach (var worker in task.Workers)
            {
                if (worker.TypeInfo == SpecialistType)
                    total += (int)(AttackPerWorker * SpecialistMultiplier);
                else
                    total += AttackPerWorker;
            }

            return total;
        }

    }
}
