﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.NodeTasks.Effects
{
    public class DefendTerritoryEffect : IEffect<Model.NodeTask>, IInitializable
    {
        /// <summary>
        /// Resources produced when fired, per worker
        /// </summary>
        public int DefencePerWorker;

        /// <summary>
        /// The specialist type that will provide a multiplier.
        /// </summary>
        public WorkerTypeInfo SpecialistType;

        /// <summary>
        /// The multiplier the specialist provides
        /// </summary>
        public float SpecialistMultiplier;

        private IHostNotify _hostNotify;

        public void Initialize(IBuildcraftContainer container)
        {
            _hostNotify = container.RetrieveInstance<IHostNotify>();
        }

        #region IEffect Members

        public void Apply(NodeTask task)
        {
            int combatBonus = CalculateDefenceContribution(task);
            task.Owner.EffectValues.Add(task.TaskId, this, combatBonus);
            RefreshDefenceValue(task);
        }

        public void Fire(NodeTask task)
        {
            //Does nothing
        }

        public void Refresh(NodeTask task)
        {
            int combatBonus = CalculateDefenceContribution(task);
            task.Owner.EffectValues.Modify(task.TaskId, this, combatBonus);
            RefreshDefenceValue(task);
        }

        public void Remove(NodeTask task)
        {
            task.Owner.EffectValues.Remove(task.TaskId, this);
            RefreshDefenceValue(task);
        }

        #endregion


        private int CalculateDefenceContribution(NodeTask task)
        {
            int total = 0;

            foreach (var worker in task.Workers)
            {
                if (worker.TypeInfo == SpecialistType)
                    total += (int)(DefencePerWorker * SpecialistMultiplier);
                else
                    total += DefencePerWorker;
            }

            return total;
        }

        private void RefreshDefenceValue(NodeTask task)
        {
            var player = task.Owner;
            player.Defence = player.EffectValues.GetTotalValueForEffect(this);
        }

    }
}
