﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.NodeTasks.Effects
{
    public class ResourceProductionEffect : IEffect<Model.NodeTask>, IInitializable
    {
        /// <summary>
        /// The type of resource produced when fired
        /// </summary>
        public ResourceTypeInfo ResourceProduced;

        /// <summary>
        /// Resources produced when fired, per worker.
        /// </summary>
        public int ProductionPerWorker;

        /// <summary>
        /// The specialist type that will provide a multiplier.
        /// </summary>
        public WorkerTypeInfo SpecialistType;

        /// <summary>
        /// The multiplier the specialist produces per 
        /// </summary>
        public float SpecialistMultiplier;


        private IHostNotify _hostNotify;

        public void Initialize(IBuildcraftContainer container)
        {
            _hostNotify = container.RetrieveInstance<IHostNotify>();
        }


        #region IEffect Members

        public void Apply(Model.NodeTask task)
        {
            //Nothing to apply
        }

        public void Fire(Model.NodeTask task)
        {
            //Calculate total production
            float production = 0;
            foreach (Worker worker in task.Workers)
            {
                if (worker.TypeInfo == SpecialistType)
                    production += this.ProductionPerWorker * SpecialistMultiplier;
                else
                    production += this.ProductionPerWorker;
            }

            //Add production to resource dictionary
            task.Owner.Resources[this.ResourceProduced] += (int)production;
        }

        public void Refresh(NodeTask task)
        {
            throw new NotImplementedException();
        }

        public void Remove(Model.NodeTask task)
        {
            //Nothing to remove
        }

        #endregion
    }
}
