﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.NodeTasks.Effects
{
    public class WorkImprovementEffect : IEffect<NodeTask>, IInitializable
    {
        private IHostNotify _hostNotify;

        public void Initialize(IBuildcraftContainer container)
        {
            _hostNotify = container.RetrieveInstance<IHostNotify>();
        }

        #region IEffect<NodeTask> Members

        public void Apply(NodeTask task)
        {
        }

        public void Fire(NodeTask task)
        {
            var player = task.Owner;
            var node = task.Node;
            var improvement = node.Improvement;
            var improvementType = improvement.TypeInfo;
            
            var production = 0;

            foreach(var worker in task.Workers)
            {
                if (worker.TypeInfo == improvementType.SpecialistWorkerType)
                {
                    production += improvementType.ResourceProductionRate * 2;
                }
                else
                {
                    production += improvementType.ResourceProductionRate;
                }
            }

            player.Resources[improvementType.ResourceProductionType] += production;
            _hostNotify.ResourcesProduced(task, improvementType.ResourceProductionType, production);
        }

        public void Refresh(NodeTask task)
        {
        }

        public void Remove(NodeTask task)
        {
        }

        #endregion
    }
}
