﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Components;
using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.PlayerActions
{
    public class AddTaskAction
    {
        /// <summary>
        /// The player this task is for.
        /// </summary>
        private Player _player;

        /// <summary>
        /// Game task manager instance.
        /// </summary>
        private PlayerManager _playerManager;


        public AddTaskAction(Player player, IContainer container)
        {
            _player = player;
            _playerManager = container.RetrieveInstance<PlayerManager>();
        }

        public TaskValidityState CanAddTask(Node node, NodeTaskTypeInfo taskType, IEnumerable<Worker> workers)
        {
            return taskType.CheckNewTaskValidity(node, _player, workers);
        }

        public TaskValidityState AddTask(Node node, NodeTaskTypeInfo taskType, IEnumerable<Worker> workers)
        {
            //Guard Condition
            var checkState = taskType.CheckNewTaskValidity(node, _player, workers);
            if (checkState != TaskValidityState.Ok)
                return checkState;

            //Add Task
            var task = _playerManager.AddNodeTask(_player, node, taskType);

            //Populate The Task
            foreach (var worker in workers)
            {
                _playerManager.AssignWorker(task, worker);
            }

            //Apply Effects
            task.ApplyEffects();
            
            #if DEBUG
            //Confirm validity in debug
            if (taskType.CheckExistingTaskValidity(task) != TaskValidityState.Ok)
                throw new BuildcraftException("Task added in invalid state.");
            #endif

            return TaskValidityState.Ok;
        }
    }

}
