﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Components;


namespace Buildcraft.Core.PlayerActions
{
    public class GetTasksAction
    {
        /// <summary>
        /// The player this task is for.
        /// </summary>
        private Player _player;

        /// <summary>
        /// Game task manager instance.
        /// </summary>
        private PlayerManager _playerManager;

        /// <summary>
        /// World
        /// </summary>
        private World _world;

        private IContainer _container;

        private List<NodeTaskTypeInfo> _internalTaskList = new List<NodeTaskTypeInfo>();

        public GetTasksAction(Player player, IContainer container)
        {
            _container = container;
            _player = player;
            _playerManager = container.RetrieveInstance<PlayerManager>();
            _world = container.RetrieveInstance<World>();
        }


        public List<NodeTaskTypeInfo> GetTasksForNode(Node node)
        {
            _internalTaskList.Clear();

            //Get tasks that can be added to the node, then add them to the internal collection
            var tasks = _container.RetrieveAllInstances<NodeTaskTypeInfo>().Where(t => t.CheckNodeValidity(node, _player) == TaskValidityState.Ok);
            _internalTaskList.AddRange(tasks);

            return _internalTaskList;
        }
    }
}
