﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

using Buildcraft.Core.Exceptions;
using Buildcraft.Core.Components;

namespace Buildcraft.Core.PlayerActions
{
    public class ModifyTaskAction
    {
        public Player Player;
        public PlayerManager PlayerManager;

        public ModifyTaskAction(Model.Player player, IContainer container)
        {
            Player = player;
            PlayerManager = container.RetrieveInstance<PlayerManager>();
        }

        public TaskValidityState CanModifyTask(NodeTask task, IEnumerable<Worker> workers)
        {
            return task.Type.CheckTaskValidity(task.Node, Player, workers, true);
        }

        public TaskValidityState ModifyTask(NodeTask task, IEnumerable<Worker> workers)
        {
            var taskType = task.Type;

            //Guard Condition
            var checkState = taskType.CheckTaskValidity(task.Node, Player, workers, true);
            if (checkState != TaskValidityState.Ok)
                return checkState;

            PlayerManager.ClearWorkers(task);

            //Populate The Task
            foreach (var worker in workers)
            {
                PlayerManager.AssignWorker(task, worker);
            }

#if DEBUG
            //Confirm validity in debug
            if (taskType.CheckExistingTaskValidity(task) != TaskValidityState.Ok)
                throw new BuildcraftException("Task added in invalid state.");
#endif

            return TaskValidityState.Ok;
        }
    }
}
