﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Components;
using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.PlayerTasks.Effects
{
    public class WorkerConsumptionEffect : IEffect<PlayerTask>, IInitializable
    {
        private PlayerManager _playerManager;
        private IHostNotify _hostNotify;

        public void Initialize(IBuildcraftContainer container)
        {
            _hostNotify = container.RetrieveInstance<IHostNotify>();
            _playerManager = container.RetrieveInstance<PlayerManager>();
        }

        #region IEffect<PlayerTask> Members

        public void Apply(PlayerTask task)
        {
        }

        public void Fire(PlayerTask task)
        {
            //Determine Cost of all workers
            ResourceValueCollection totals = new ResourceValueCollection();

            //Calculate totals and run cost check
            PopulateConsumptionTotals(task.Owner, totals);
            bool canAfford = CanAffordCheck(task.Owner, totals);

            while(!canAfford)
            {
                //If we don't have enough, kill a worker
                var worker = GetDeadWorker(task.Owner);

                //Calculate affordability
                PopulateConsumptionTotals(task.Owner, totals);
                canAfford = CanAffordCheck(task.Owner, totals);
            }

            //Deduct Resources
            foreach (var resource in totals)
            {
                task.Owner.Resources[resource] -= totals[resource];
            }

        }

        public void Refresh(PlayerTask task)
        {
        }

        public void Remove(PlayerTask task)
        {
        }

        #endregion


        private bool CanAffordCheck(Player player, ResourceValueCollection totals)
        {
            foreach (var resource in totals)
            {
                if (player.Resources[resource] < totals[resource])
                    return false;
            }

            return true;
        }

        private void PopulateConsumptionTotals(Player player, ResourceValueCollection totals)
        {
            totals.Clear();

            foreach (var worker in player.AllWorkers)
            {
                //Add all resources for worker consumption
                foreach (var resource in worker.TypeInfo.Consumption)
                {
                    totals[resource] += worker.TypeInfo.Consumption[resource];
                }
            }
        }

        private Worker GetDeadWorker(Player player)
        {
            Worker worker;
            
            //Use unassigned workers as a preference
            if (player.UnassignedWorkers.Count > 0)
            {
                worker = player.UnassignedWorkers
                    .OrderBy(w => w.TypeInfo.Rank)
                    .First();

                _playerManager.RemoveWorker(worker);
            }
            else
            {
                worker = player.AllWorkers
                    .OrderBy(w => w.TypeInfo.Rank)
                    .First();

                _playerManager.UnassignWorker(worker.Task, worker);
                _playerManager.RemoveWorker(worker);
            }

            return worker;
        }
    }
}
