﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Exceptions;
using Buildcraft.Core.Model;

namespace Buildcraft.Core.PlayerTasks
{
    public class PlayerTaskTypeInfo : IInitializable
    {
        public string PlayerTaskTypeCode;

        public string EffectCode;

        public int TaskDuration;

        public bool Recycle;

        private IEffect<PlayerTask> Effect;



        public PlayerTaskTypeInfo(string playerTaskTypeCode, string effectCode)
        {
            PlayerTaskTypeCode = playerTaskTypeCode;
            EffectCode = effectCode;
        }

        public void Initialize(IBuildcraftContainer container)
        {
            //Condition = container.RetrieveInstance<ICondition>(ConditionCode);
            Effect = container.RetrieveInstance<IEffect<PlayerTask>>(EffectCode);
            //HostNotify = container.RetrieveInstance<IHostNotify>();
        }


        private TaskValidityState CheckExistingTaskValidity(PlayerTask task)
        {
            return TaskValidityState.Ok;
        }


        public TaskState Tick(PlayerTask task, int progress)
        {
            //Check task is valid
            if (CheckExistingTaskValidity(task) != TaskValidityState.Ok)
                return TaskState.Failed;

            //Progress Task
            task.Progress += progress;

            bool complete = (task.Progress >= this.TaskDuration);

            TaskState result;

            //Check for completeness
            if (!complete)
                result = TaskState.Working;
            else
            {
                result = this.Recycle ?
                    TaskState.Recycling :
                    TaskState.Complete;
            }

            return result;
        }

        public void ApplyEffects(PlayerTask task)
        {
            if (CheckExistingTaskValidity(task) != TaskValidityState.Ok)
                throw new BuildcraftException("Cannot apply effect for invalid task.");

            Effect.Apply(task);
        }

        public void RefreshEffects(PlayerTask task)
        {
            if (CheckExistingTaskValidity(task) != TaskValidityState.Ok)
                throw new BuildcraftException("Cannot refresh effect for invalid task.");

            Effect.Refresh(task);
        }

        public void FireEffects(PlayerTask task)
        {
            if (CheckExistingTaskValidity(task) != TaskValidityState.Ok)
                throw new BuildcraftException("Cannot fire effect for invalid task.");

            Effect.Fire(task);
        }

        public void RemoveEffects(PlayerTask task)
        {
            Effect.Remove(task);
        }
    }
}
