﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.UnitTasks;
using Buildcraft.Core.Interfaces;

namespace Buildcraft.Core.TaskProcessors
{
    public class TaskProductionProcessor : ITaskProcessor
    {
        private List<IUnitTaskStateBucketProduction> _stateBuckets = new List<IUnitTaskStateBucketProduction>();

        #region ITaskProcessor Members

        public void RegisterStateBucket(IUnitTaskStateBucket bucket)
        {
            if (bucket == null)
                throw new ArgumentNullException();

            if (bucket is IUnitTaskStateBucketProduction
                || bucket is IUnitTaskStateBucketProduction)
            {
                _stateBuckets.Add((IUnitTaskStateBucketProduction)bucket);

                //NOTE: Could sort the tasks by priority?
            }
        }

        public void UnregisterStateBucket(IUnitTaskStateBucket bucket)
        {
            if (bucket == null)
                throw new ArgumentNullException();

            if (bucket.TaskInfo is IUnitTaskStateBucketProduction)
                _stateBuckets.Remove((IUnitTaskStateBucketProduction)bucket);
        }

        public void RegisterState(IUnitTaskStateItem state)
        {
            //No Action - Ignore
        }

        public void UnregisterState(IUnitTaskStateItem state)
        {
            //No Action - Ignore
        }

        public void Process()
        {
            //Add Process to production
            foreach (var task in _stateBuckets)
            {
                var owner = task.Owner;

                if (task.IsActive)
                    owner.Resources.Add(task.Production);
            }
        }

        #endregion
    }
}
