﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Interfaces;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Exceptions;
using Buildcraft.Core.UnitTasks.WorkImprovement;
using Buildcraft.Core.UnitTasks;
using Buildcraft.Core.Interfaces.UnitTasks.TaskInfo;

namespace Buildcraft.Core.TaskProcessors
{
    /// <summary>
    /// Processor for updating consumption resource production
    /// </summary>
    public class TaskRunningCostProcessor : ITaskProcessor
    {
        private List<IUnitTaskStateBucketRunningCost> _stateBuckets = new List<IUnitTaskStateBucketRunningCost>();

        #region ITaskProcessor Members

        public void RegisterStateBucket(IUnitTaskStateBucket bucket)
        {
            if (bucket == null)
                throw new ArgumentNullException();

            if (bucket is IUnitTaskStateBucketRunningCost
                || bucket is IUnitTaskStateBucketRunningCost)
            {
                _stateBuckets.Add((IUnitTaskStateBucketRunningCost)bucket);
            
                //TODO: Sort the tasks by priority?
            }
        }

        public void UnregisterStateBucket(IUnitTaskStateBucket bucket)
        {
            if (bucket == null)
                throw new ArgumentNullException();

            if (bucket.TaskInfo is IUnitTaskStateBucketRunningCost)
                _stateBuckets.Remove((IUnitTaskStateBucketRunningCost)bucket);
        }

        public void RegisterState(IUnitTaskStateItem state)
        {
            //No Action - Ignore
        }

        public void UnregisterState(IUnitTaskStateItem state)
        {
            //No Action - Ignore
        }

        public void Process()
        {
            //Deduct Costs, and determine task activity
            foreach (var task in _stateBuckets)
            {
                //Unpack
                var owner = task.Owner;
                var availableResources = owner.Resources;
                var runningCost = task.RunningCost;

                //Determine if can afford, and deduct cost appropriately
                bool canAfford = availableResources.CanSubtract(runningCost);
                if (canAfford)
                    availableResources.Subtract(runningCost);

                //Setup Paid For Flag
                task.IsPaidFor = canAfford;
            }
        }

        #endregion
    }
}
