﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Interfaces;
using Buildcraft.Core.Model;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Components;

namespace Buildcraft.Core.TaskProcessors
{
    public class UnitSpawnProcessor : ITaskProcessor, IInitializable
    {
        private World _world;
        private PlayerManager _playerManager;

        private ResourceValueCollection _spawnCost = new ResourceValueCollection();
        private UnitTypeInfo _spawnedUnitType;


        public ResourceValueCollection SpawnCost
        {
            get { return _spawnCost; }
        }

        //TODO: Find a different route to inject this
        public UnitTypeInfo SpawnedUnitType
        {
            get { return _spawnedUnitType; }
            set { _spawnedUnitType = value; }
        }

        #region IInitializable Members

        public void Initialize(IBuildcraftContainer container)
        {
            _world = container.RetrieveInstance<World>();
            _playerManager = container.RetrieveInstance<PlayerManager>();
        }

        #endregion

        #region ITaskProcessor Members

        public void RegisterStateBucket(UnitTasks.IUnitTaskStateBucket bucket)
        {
            
        }

        public void UnregisterStateBucket(UnitTasks.IUnitTaskStateBucket bucket)
        {
            
        }


        public void RegisterState(UnitTasks.IUnitTaskStateItem state)
        {
            
        }

        public void UnregisterState(UnitTasks.IUnitTaskStateItem state)
        {
            
        }


        public void Process()
        {
            foreach (var player in _world.Players)
            {
                ProcessPlayer(player);
            }
        }

        #endregion

        private void ProcessPlayer(Player owner)
        {
            //Unpack
            var resources = owner.Resources;

            //Check if owner can afford
            if (!resources.CanSubtract(_spawnCost))
                return;

            resources.Subtract(_spawnCost);
            _playerManager.AddUnit(owner, _spawnedUnitType);
            //TODO: Notify?
        }
    }
}
