﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

namespace Buildcraft.Core.UnitTasks.AttackNode
{
    public interface IAttackNodeTaskStateBucket :
        IUnitTaskStateBucketRunningCost,
        IUnitTaskStateBucketNodeAttack
    {
    }

    public class AttackNodeTaskStateBucket : UnitTaskStateBucketBase, IAttackNodeTaskStateBucket
    {
        private ResourceValueCollection _runningCost;
        private float _attackPower;

        public AttackNodeTaskStateBucket(AttackNodeTaskTypeInfo taskInfo, Node node, Player player)
            :base(taskInfo, node, player)
        {
            _runningCost = new ResourceValueCollection();
            _attackPower = 0;
        }

        #region IUnitTaskStateBucketRunningCost Members

        public ResourceValueCollection RunningCost
        {
            get { return _runningCost; }
        }

        public bool IsPaidFor
        {
            get;
            set;
        }

        #endregion

        #region IUnitTaskStateBucketNodeAttack Members

        public float Power
        {
            get { return _attackPower; }
        }

        #endregion

        protected override void InternalRecalculate()
        {
            _runningCost.Clear();
            _attackPower = 0;

            //Recalculate All Children running this type
            foreach (AttackNodeTaskState taskState in _taskStates)
            {
                _attackPower += taskState.Power;
                _runningCost.Add(taskState.RunningCost);
            }
        }
    }
}
