﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.UnitTasks.AttackNode
{
    public interface IAttackNodeTaskTypeInfo :
        IUnitTaskInfo
    {
    }

    public class AttackNodeTaskTypeInfo : UnitTaskInfoBase, IAttackNodeTaskTypeInfo
    {
        private Dictionary<UnitTypeInfo, int> _combatMapping;
        private Dictionary<UnitTypeInfo, ResourceValueCollection> _consumptionMapping;

        public AttackNodeTaskTypeInfo()
            : base("AttackNode", UnitTaskCategory.AttackNode)
        { }

        public void Setup(Dictionary<UnitTypeInfo, int> combatMapping, Dictionary<UnitTypeInfo, ResourceValueCollection> consumptionMapping)
        {
            _combatMapping = combatMapping;
            _consumptionMapping = consumptionMapping;
        }


        public override IUnitTaskStateItem CreateTaskState(Model.Node node, Model.Unit unit)
        {
            var state = new AttackNodeTaskState(this, node, unit);
            state.Recalculate();
            return state;
        }

        public override IUnitTaskStateBucket CreateTaskStateBucket(Model.Node node, Model.Player player)
        {
            return new AttackNodeTaskStateBucket(this, node, player);
        }


        public int CalculatePower(Model.Node node, Unit unit)
        {
            var unitType = unit.TypeInfo;

            if (!_combatMapping.ContainsKey(unitType))
                throw new BuildcraftException("Unmapped unit type: " + unitType);

            return _combatMapping[unitType];
        }

        public void CalculatorRunningCost(Model.Node node, Unit unit, Model.ResourceValueCollection results)
        {
            var unitType = unit.TypeInfo;

            if (!_consumptionMapping.ContainsKey(unitType))
                throw new BuildcraftException("Unmapped unit type: " + unitType);

            results.Add(_consumptionMapping[unitType]);
        }
    }

}
