﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

namespace Buildcraft.Core.UnitTasks.DefendNode
{
    public interface IDefendNodeTaskStateBucket :
        IUnitTaskStateBucketRunningCost,
        IUnitTaskStateBucketNodeDefence
    {
    }

    public class DefendNodeTaskStateBucket : UnitTaskStateBucketBase, IDefendNodeTaskStateBucket
    {
        private ResourceValueCollection _runningCost;
        private float _defencePower;

        public DefendNodeTaskStateBucket(DefendNodeTaskTypeInfo taskInfo, Node node, Player owner)
            : base(taskInfo, node, owner)
        {
            _runningCost = new ResourceValueCollection();
            _defencePower = 0;
        }

        #region IUnitTaskStateBucketRunningCost Members

        public Model.ResourceValueCollection RunningCost
        {
            get { return _runningCost; }
        }

        public bool IsPaidFor
        {
            get;
            set;
        }

        #endregion

        #region IUnitTaskStateBucketNodeDefence Members

        public float Power
        {
            get { return _defencePower; }
        }

        #endregion

        protected override void InternalRecalculate()
        {
            _runningCost.Clear();
            _defencePower = 0;

            //Recalculate All Children running this type
            foreach (DefendNodeTaskState taskState in _taskStates)
            {
                _defencePower += taskState.Power;
                _runningCost.Add(taskState.RunningCost);
            }
        }
    }
}
