﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Exceptions;

namespace Buildcraft.Core.UnitTasks.DefendNode
{    
    public interface IDefendNodeTaskTypeInfo :
        IUnitTaskInfo
    {
    }

    public class DefendNodeTaskTypeInfo : UnitTaskInfoBase, IDefendNodeTaskTypeInfo
    {
        private Dictionary<UnitTypeInfo, int> _combatMapping;
        private Dictionary<UnitTypeInfo, ResourceValueCollection> _consumptionMapping;

        public DefendNodeTaskTypeInfo()
            : base("DefendNode", UnitTaskCategory.DefendNode)
        {
        }

        public override IUnitTaskStateItem CreateTaskState(Node node, Unit unit)
        {
            var state = new DefendNodeTaskState(this, node, unit);
            state.Recalculate();
            return state;
        }

        public override IUnitTaskStateBucket CreateTaskStateBucket(Node node, Player player)
        {
            return new DefendNodeTaskStateBucket(this, node, player);
        }

        public void Setup(Dictionary<UnitTypeInfo, int> combatMapping, Dictionary<UnitTypeInfo, ResourceValueCollection> consumptionMapping)
        {
            _combatMapping = combatMapping;
            _consumptionMapping = consumptionMapping;
        }

        internal void CalculatorRunningCost(Node node, Unit unit, ResourceValueCollection results)
        {
            var unitType = unit.TypeInfo;

            if (!_consumptionMapping.ContainsKey(unitType))
                throw new BuildcraftException("Unmapped unit type: " + unitType);

            results.Add(_consumptionMapping[unitType]);
        }

        internal int CalculatePower(Node node, Unit unit)
        {
            var unitType = unit.TypeInfo;

            if (!_combatMapping.ContainsKey(unitType))
                throw new BuildcraftException("Unmapped unit type: " + unitType);

            return _combatMapping[unitType];
        }
    }
}
