﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

namespace Buildcraft.Core.UnitTasks
{
    public interface IUnitTaskInfo
    {
        /// <summary>
        /// The task code for the task.
        /// </summary>
        string TaskCode { get; }

        /// <summary>
        /// The category of the task.
        /// </summary>
        UnitTaskCategory TaskCategory { get; }


        /// <summary>
        /// Get a new instance of a task state for a specific node and unit.
        /// </summary>
        /// <param name="node"></param>
        /// <param name="unit"></param>
        /// <returns></returns>
        IUnitTaskStateItem CreateTaskState(Node node, Unit unit);

        /// <summary>
        /// Get a new task state bucket for the specified node and owner
        /// </summary>
        /// <param name="node"></param>
        /// <param name="player"></param>
        /// <returns></returns>
        IUnitTaskStateBucket CreateTaskStateBucket(Node node, Player player);
    }

    public abstract class UnitTaskInfoBase : IUnitTaskInfo
    {
        protected string _taskCode;
        protected UnitTaskCategory _taskCategory;
        
        protected UnitTaskInfoBase(string taskCode, UnitTaskCategory category)
        {
            _taskCode = taskCode;
            _taskCategory = category;
        }


        #region IUnitTaskInfo Members

        public string TaskCode
        {
            get { return _taskCode; }
        }

        public UnitTaskCategory TaskCategory
        {
            get { return _taskCategory; }
        }

        public abstract IUnitTaskStateItem CreateTaskState(Node node, Unit unit);

        public abstract IUnitTaskStateBucket CreateTaskStateBucket(Node node, Player player);

        #endregion
    }
}
