﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;

namespace Buildcraft.Core.UnitTasks
{
    public interface IUnitTaskStateItem
    {
        /// <summary>
        /// The TaskInfo that this task state belongs too.
        /// </summary>
        IUnitTaskInfo TaskInfo { get; }


        /// <summary>
        /// The player that owns this task state bucket.
        /// </summary>
        Player Owner { get; }

        /// <summary>
        /// The node this task state is running on.
        /// </summary>
        Node Node { get; }

        /// <summary>
        /// The Unit this task state belongs too.
        /// </summary>
        Unit Unit { get; }


        /// <summary>
        /// Recalculate the value of the task state.
        /// </summary>
        void Recalculate();
    }

    public abstract class UnitTaskStateItemBase<T> : IUnitTaskStateItem
        where T : IUnitTaskInfo
    {
        protected T _taskInfo;
        protected Node _node;
        protected Unit _unit;
        protected Player _owner;

        protected UnitTaskStateItemBase(T taskInfo, Node node, Unit unit)
        {
            _taskInfo = taskInfo;
            _node = node;
            _unit = unit;
            _owner = unit.Owner;
        }

        /// <summary>
        /// The TaskInfo that this task state belongs too.
        /// </summary>
        public IUnitTaskInfo TaskInfo
        {
            get { return _taskInfo; }
        }

        /// <summary>
        /// The node this task state is running on.
        /// </summary>
        public Node Node
        {
            get { return _node; }
        }

        /// <summary>
        /// The Unit this task state belongs too.
        /// </summary>
        public Unit Unit
        {
            get { return _unit; }
        }

        /// <summary>
        /// The player this task state belongs too.
        /// </summary>
        public Player Owner
        {
            get { return _owner; }
        }

        /// <summary>
        /// Recalculate the task state
        /// </summary>
        public virtual void Recalculate()
        {
            //Default: Nothing
        }
    }

}
