﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Interfaces.UnitTasks.TaskInfo;

namespace Buildcraft.Core.UnitTasks.WorkImprovement
{
    public interface IWorkImprovementTaskState :
        IUnitTaskStateItemRunningCost,
        IUnitTaskStateItemProduction
    {
    }

    public class WorkImprovementTaskState : UnitTaskStateItemBase<WorkImprovementTaskTypeInfo>, IWorkImprovementTaskState
    {
        private ResourceValueCollection _runningCost;
        private ResourceValueCollection _productionCost;

        public WorkImprovementTaskState(WorkImprovementTaskTypeInfo taskInfo, Node node, Unit unit)
            :base(taskInfo, node, unit)
        {
            _runningCost = new ResourceValueCollection();
            _productionCost = new ResourceValueCollection();
        }

        public override void Recalculate()
        {
            _taskInfo.CalculateProduction(_node, _unit, _productionCost);
            _taskInfo.CalculatorRunningCost(_node, _unit, _runningCost);
        }

        #region IUnitTaskStateRunningCost Members

        public ResourceValueCollection RunningCost
        {
            get { return _runningCost; }
        }

        public bool IsPaidFor { get; set; }

        #endregion

        #region IUnitTaskStateProduction Members

        public ResourceValueCollection Production
        {
            get { return _productionCost; }
        }

        public bool IsActive
        {
            get { return IsPaidFor; }
        }

        #endregion
    }
}
