﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Exceptions;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Interfaces.UnitTasks.TaskInfo;

namespace Buildcraft.Core.UnitTasks.WorkImprovement
{
    public interface IWorkImprovementTaskStateBucket : 
        IUnitTaskStateBucketRunningCost,
        IUnitTaskStateBucketProduction
    {
    }

    public class WorkImprovementTaskStateBucket : UnitTaskStateBucketBase, IWorkImprovementTaskStateBucket
    {
        private ResourceValueCollection _runningCost;
        private ResourceValueCollection _productionCost;

        public WorkImprovementTaskStateBucket(WorkImprovementTaskTypeInfo taskInfo, Node node, Player owner)
            :base(taskInfo, node, owner)
        {
            _runningCost = new ResourceValueCollection();
            _productionCost = new ResourceValueCollection();
        }

        #region IUnitTaskStateRunningCost Members

        public ResourceValueCollection RunningCost
        {
            get { return _runningCost; }
        }

        public bool IsPaidFor { get; set; }

        #endregion

        #region IUnitTaskStateProduction Members

        public ResourceValueCollection Production
        {
            get { return _productionCost; }
        }

        public bool IsActive
        {
            get { return IsPaidFor; }
        }

        #endregion


        protected override void InternalRecalculate()
        {
            _runningCost.Clear();
            _productionCost.Clear();

            //Recalculate All Children running this type
            foreach (WorkImprovementTaskState taskState in _taskStates)
            {
                _productionCost.Add(taskState.Production);
                _runningCost.Add(taskState.RunningCost);
            }
        }
    }
}
