﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Buildcraft.Core.Model;
using Buildcraft.Core.Interfaces.UnitTasks;
using Buildcraft.Core.Interfaces.UnitTasks.TaskInfo;
using Buildcraft.Core.Interfaces;
using Buildcraft.Core.Exceptions;


namespace Buildcraft.Core.UnitTasks.WorkImprovement
{
    public interface IWorkImprovementTaskTypeInfo :
        IUnitTaskInfo,
        IUnitTaskNodeEffectTypeConstraint,
        IUnitTaskImprovementTypeConstraint,
        IUnitTaskUnitTypeConstraint
    {
    }

    public class WorkImprovementTaskTypeInfo : UnitTaskInfoBase, IWorkImprovementTaskTypeInfo
    {
        private IEntityTypeCheck<ImprovementTypeInfo> _improvementTypeCheck;
        private IEntityTypeCheck<UnitTypeInfo> _unitTypeCheck;
        private IEntityTypeCheck<NodeEffectTypeInfo> _effectTypeCheck;

        private Dictionary<UnitTypeInfo, ResourceValueCollection> _productionMapping;
        private Dictionary<UnitTypeInfo, ResourceValueCollection> _consumptionMapping;


        public WorkImprovementTaskTypeInfo()
            :base("WorkImprovement", UnitTaskCategory.WorkImprovement)
        {
        }

        public void Setup(Dictionary<UnitTypeInfo, ResourceValueCollection> productionMapping, Dictionary<UnitTypeInfo, ResourceValueCollection> consumptionMapping)
        {
            _productionMapping = productionMapping;
            _consumptionMapping = consumptionMapping;
        }


        #region IUnitTaskInfo Members

        public override IUnitTaskStateItem CreateTaskState(Node node, Unit unit)
        {
            var state = new WorkImprovementTaskState(this, node, unit);
            state.Recalculate();
            return state;
        }

        public override IUnitTaskStateBucket CreateTaskStateBucket(Node node, Player player)
        {
            return new WorkImprovementTaskStateBucket(this, node, player);
        }

        #endregion

        #region IUnitTaskNodeEffectTypeConstraint Members

        public IEntityTypeCheck<NodeEffectTypeInfo> EffectTypeCheck
        {
            get { return _effectTypeCheck; }
        }

        #endregion

        #region IUnitTaskImprovementTypeConstraint Members

        public IEntityTypeCheck<ImprovementTypeInfo> ImprovementTypeCheck
        {
            get { return _improvementTypeCheck; }
        }

        #endregion

        #region IUnitTaskUnitTypeConstraint Members

        public IEntityTypeCheck<UnitTypeInfo> UnitTypeCheck
        {
            get { return _unitTypeCheck; }
        }

        #endregion


        public void CalculateProduction(Model.Node node, Unit unit, Model.ResourceValueCollection results)
        {
            var unitType = unit.TypeInfo;

            if (!_productionMapping.ContainsKey(unitType))
                throw new BuildcraftException("Unmapped unit type: " + unitType);

            results.Add(_productionMapping[unitType]);
        }

        public void CalculatorRunningCost(Model.Node node, Unit unit, Model.ResourceValueCollection results)
        {
            var unitType = unit.TypeInfo;

            if (!_consumptionMapping.ContainsKey(unitType))
                throw new BuildcraftException("Unmapped unit type: " + unitType);

            results.Add(_consumptionMapping[unitType]);
        }
    }
}
